## This file is part of HamsiManager.
## 
## Copyright (c) 2010 Murat Demir <mopened@gmail.com>      
##
## Hamsi Manager is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## Hamsi Manager is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with HamsiManager; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


isAvailable = False
try:
    from musicbrainz2 import webservice, model, utils
    from musicbrainz2.webservice import Query, ArtistFilter, WebServiceError, ReleaseFilter, TrackFilter
    isAvailable = True
except:pass
from MyObjects import *
import Dialogs
import time
import Universals
pluginName = "MusicBrainz"

class Search(MDialog):

    def __init__(self, _parent, _isCheckSingleFile=False, _searchDepth=3):
        MDialog.__init__(self, _parent)
        if Universals.isActivePyKDE4==True:
            self.setButtons(MDialog.NoDefault)
        self.isCheckSingleFile = _isCheckSingleFile
        self.searchDepth = _searchDepth
        self.nullSongs, self.falseSongs, self.trueSongs, self.songsOfArtist, self.songsOfAlbum, self.cbTags, self.leTags, self.incorrectSongs = [], [], [], [], [], [], [], []
        self.searchedArtists, self.searchedTitles, self.searchedAlbums = [[], []], [[], []], [[], []]
        self.searchedAlbumsOfArtist, self.searchedSongsOfAlbum, self.searchedDetailsOfArtist, self.searchedDetailsOfAlbum = [[], []], [[], []], [[], []], [[], []]
        self.showSuggest(True)
        self.isAlterAlbum = False
        self.isArtistImportant = True
        self.isArtistChanged = False
        self.isAlterArtist = True
        self.setModal(True)
        self.prgbState = MProgressBar()
        self.prgbAllState = MProgressBar()
        self.prgbStateLabel = MLabel(translate("SearchEngines", "Current Proccess"))
        self.prgbAllStateLabel = MLabel(translate("SearchEngines", "General"))
        self.pbtnApply = MPushButton(translate("SearchEngines", "Apply"))
        self.pbtnApply.setMaximumWidth(120)
        self.pbtnApply.setVisible(False)
        self.pbtnCancel = MPushButton(translate("SearchEngines", "Cancel"))
        self.pbtnCancel.setMaximumWidth(120)
        self.pbtnClose = MPushButton(translate("SearchEngines", "Close"))
        self.pbtnClose.setMaximumWidth(120)
        self.pbtnClose.setVisible(False)
        MObject.connect(self.pbtnApply,SIGNAL("clicked()"),self.apply)
        MObject.connect(self.pbtnClose,SIGNAL("clicked()"),self.close)
        MObject.connect(self.pbtnCancel, SIGNAL("clicked()"), Universals.cancelThreadAction)
        pnlMain = MWidget(self)
        self.saPanel = MScrollArea(pnlMain)
        self.vblPanel = MVBoxLayout()
        self.vblPanel.setAlignment(Mt.AlignHCenter)
        self.vblPanel.setAlignment(Mt.AlignTop)
        self.vblPanel.addWidget(self.prgbStateLabel)
        self.vblPanel.addWidget(self.prgbState)
        self.vblPanel.addWidget(self.prgbAllStateLabel)
        self.vblPanel.addWidget(self.prgbAllState)
        self.pnlPanel = MWidget(pnlMain)
        self.pnlPanel.setLayout(self.vblPanel)
        self.pnlPanel.setFixedSize(595,100)
        self.saPanel.setWidget(self.pnlPanel)
        self.saPanel.setFrameShape(MFrame.StyledPanel)
        self.saPanel.setAlignment(Mt.AlignHCenter)
        self.saPanel.setFixedSize(645,110)
        vblBox = MVBoxLayout(pnlMain)
        vblBox.addWidget(self.saPanel)
        hblBox = MHBoxLayout()
        hblBox.addStretch(2)
        hblBox.addWidget(self.pbtnCancel)
        hblBox.addWidget(self.pbtnClose)
        hblBox.addWidget(self.pbtnApply)
        vblBox.addLayout(hblBox)
        if Universals.isActivePyKDE4==True:
            self.setMainWidget(pnlMain)
        else:
            self.setLayout(vblBox)
        self.setWindowTitle(translate("SearchEngines", "Searching Information On The Internet!.."))
        self.setFixedSize(670,160)
        self.setAttribute(Mt.WA_DeleteOnClose)
        self.show()
        self.connect(self, SIGNAL("changedProgressBarValue"), self.changeProgressBarValue)
        import MyThread
        myProcs = MyThread.MyThread(self.startSearch, self.finishSearch)
        myProcs.start()
        
    def closeEvent(self, _event):
        if Universals.isContinueThreadAction():
            Universals.cancelThreadAction()
            _event.ignore()
        
    def changeProgressBarValue(self, _progressBarName, _value):
        if _progressBarName=="AllState":
            self.prgbAllState.setValue(_value)
            self.prgbAllState.repaint()
        elif _progressBarName=="State":
            self.prgbState.setValue(_value)
            self.prgbState.repaint()
        
    def startSearch(self):
        if self.isCheckSingleFile:
            self.prgbAllState.setRange(0,1)
            self.rows = list(range(Universals.MainWindow.Table.currentRow(), Universals.MainWindow.Table.currentRow()+1))
            self.heightValue = 150
        else:
            self.prgbAllState.setRange(0,Universals.MainWindow.Table.rowCount())
            self.rows = list(range(Universals.MainWindow.Table.rowCount()))
            if Universals.MainWindow.Table.rowCount()<7:
                self.heightValue = 300
            else:
                self.heightValue = 500
        tagsOfSongs = []
        for rowNo in self.rows:
            tagsOfSongs.append([str(Universals.MainWindow.Table.item(rowNo,2).text()),
                               str(Universals.MainWindow.Table.item(rowNo,3).text()),
                               str(Universals.MainWindow.Table.item(rowNo,4).text()), rowNo])
        Universals.startThreadAction()
        for tagsOfSong in tagsOfSongs:
            isContinueThreadAction = Universals.isContinueThreadAction()
            if isContinueThreadAction:
                try:
                    self.checkIt(tagsOfSong, self.searchDepth)
                except webservice.WebServiceError as errorDetails:
                    Dialogs.showError(translate("SearchEngines", "An Error Occured"),
                                str(translate("SearchEngines", "Please retry the process.<br>If you receive the same error, please try the other search engines.<br><b>Error details:</b><br>%s")) % (str(errorDetails)))
                except ValueError as errorDetails:
                    Dialogs.showError(translate("SearchEngines", "An Error Occured"),
                                str(translate("SearchEngines", "Fetching information for the music file that caused the error is canceled.<br>If you receive the same error, please try the other search engines.<br><b>Error details:</b><br>%s")) % (str(errorDetails)))
                    self.incorrectSongs.append([[Universals.MainWindow.Table.item(tagsOfSong[3],1).text()], [tagsOfSong[0]], [tagsOfSong[1]], [tagsOfSong[2]], tagsOfSong[3]])
                self.emit(SIGNAL("changedProgressBarValue"), "AllState", tagsOfSong[3]+1)
        Universals.finishThreadAction()
        return True
            
    def finishSearch(self, _isContinue):
        if _isContinue:
            self.prgbState.setVisible(False)
            self.prgbAllState.setVisible(False)
            self.prgbStateLabel.setVisible(False)
            self.prgbAllStateLabel.setVisible(False)
            self.showInList()
            self.pbtnCancel.setVisible(False)
            self.pbtnClose.setVisible(True)
            self.pbtnApply.setVisible(True)
            self.setMinimumSize(670,self.heightValue+50)
            self.saPanel.setFixedSize(645,self.heightValue)
        
    def checkIt(self, _tagsOfSong, _searchDepth):
        self.prgbState.setRange(0,100)
        if _tagsOfSong[1].strip()!="":
            searchType = "Title"
            self.emit(SIGNAL("changedProgressBarValue"), "State", 2)
        elif _tagsOfSong[2].strip()!="":
            searchType = "Album"
            self.emit(SIGNAL("changedProgressBarValue"), "State", 10)
        elif _tagsOfSong[0].strip()!="":
            searchType = "Artist"
            self.emit(SIGNAL("changedProgressBarValue"), "State", 20)
        else:
            self.emit(SIGNAL("changedProgressBarValue"), "State", 100)
            self.nullSongs.append([[self.parent().item(_tagsOfSong[3],1).text()], [_tagsOfSong[0]], [_tagsOfSong[1]], [_tagsOfSong[2]], _tagsOfSong[3]])
            return
        if searchType=="Title":
            titles = self.getSongsFromTitle(_tagsOfSong[1].strip())
            if len(titles)!=0:
                self.emit(SIGNAL("changedProgressBarValue"), "State", 90)
                for title in titles:
                    if str(title[2]).lower()==str(_tagsOfSong[1]).lower():
                        if str(title[5]).lower()==str(_tagsOfSong[2]).lower():
                            if str(title[7]).lower()==str(_tagsOfSong[0]).lower():
                                self.trueSongs.append([_tagsOfSong[0], _tagsOfSong[1], _tagsOfSong[2], _tagsOfSong[3], _tagsOfSong[0], _tagsOfSong[1], _tagsOfSong[2]])
                                return
                artistNames, albumNames, titleNames=[], [], []
                for title in titles:
                    if artistNames.count(title[7])==0:
                        artistNames.append(title[7])
                    if titleNames.count(title[2])==0:
                        titleNames.append(title[2])
                    if albumNames.count(title[5])==0:
                        albumNames.append(title[5])
                self.falseSongs.append([artistNames, titleNames, albumNames, _tagsOfSong[3], _tagsOfSong[0], _tagsOfSong[1], _tagsOfSong[2]])
            else:
                searchType = "Album"
        if searchType=="Album":
            albums = self.getAlbumsFromAlbum(_tagsOfSong[2].strip())
            if len(albums)!=0:
                self.emit(SIGNAL("changedProgressBarValue"), "State", 60)
                artistNames, albumNames, titleNames=[], [], []
                for album in albums:
                    if artistNames.count(album[7])==0:
                        artistNames.append(album[7])
                        albumNames.append([])
                        titleNames.append([])
                        artistNo = -1
                    else:
                        artistNo = artistNames.index(album[7])
                    albumNames[artistNo].append(album[2])
                    titleNames[artistNo].append([])
                    titles = self.getSongsOfAlbum(album[1])
                    self.emit(SIGNAL("changedProgressBarValue"), "State", 90)
                    if len(titles)!=0:
                        for title in titles:
                            titleNames[artistNo][-1].append(title[1])
                self.songsOfAlbum.append([artistNames,titleNames,albumNames, _tagsOfSong[3],_tagsOfSong[0], _tagsOfSong[1],_tagsOfSong[2]])   
            else:
                searchType = "Artist"
        if searchType=="Artist":
            artists = self.getArtistsFromArtist(_tagsOfSong[0].strip())
            if len(artists)!=0:
                self.emit(SIGNAL("changedProgressBarValue"), "State", 40)
                artistNames, albumNames, titleNames=[], [], []
                for artist in artists:
                    artistNames.append(artist[2])
                    albumNames.append([])
                    titleNames.append([])
                    albums = self.getAlbumsOfArtist(artist[1], _searchDepth)
                    if len(albums)!=0:
                        self.emit(SIGNAL("changedProgressBarValue"), "State", 65)
                        for album in albums:
                            albumNames[-1].append(album[1])
                            titleNames[-1].append([])
                            titles = self.getSongsOfAlbum(album[0])
                            if len(titles)!=0:
                                self.emit(SIGNAL("changedProgressBarValue"), "State", 90)
                                for title in titles:
                                    titleNames[-1][-1].append(title[1])
                self.songsOfArtist.append([artistNames,titleNames,albumNames, _tagsOfSong[3],_tagsOfSong[0], _tagsOfSong[1],_tagsOfSong[2]])
            else:
                self.nullSongs.append([[self.parent().item(_tagsOfSong[3],1).text()], [_tagsOfSong[0]], [_tagsOfSong[1]], [_tagsOfSong[2]], _tagsOfSong[3]])

    def getArtistsFromArtist(self, _artistName):
        """Returns artists that are similar to the one selected.
        Returned[x][0]:Match             Returned[x][1]:Artist ID
        Returned[x][2]:Name              Returned[x][3]:Nickname"""
        try:
            return self.searchedArtists[1][self.searchedArtists[0].index(_artistName)]
        except:pass
        self.searchedArtists[0].append(_artistName)
        values, controlValue=[], 1
        while controlValue>0 and controlValue<4:
            try:
                returnedValues = Query().getArtists(ArtistFilter(_artistName, limit=5))
                controlValue=0
            except webservice.WebServiceError as errorDetails:
                if str(errorDetails)[:15]=="HTTP Error 503:":
                    time.sleep(controlValue)
                    controlValue+=1
                else:
                    raise ValueError(errorDetails)
        for result in returnedValues:
            values.append([result.score, result.artist.id, result.artist.name, result.artist.sortName])
        self.searchedArtists[1].append(values)
        return values

    def getAlbumsFromAlbum(self, _albumName):
        """Returns albums that are similar to the one selected.
        Returned[x][0]:Match            Returned[x][1]:Id
        Returned[x][2]:Name             Returned[x][3]:Asin
        Returned[x][4]:Text             Returned[x][5]:Genre
        Returned[x][6]:Artist Id	      Returned[x][7]:Artist Name"""
        try:
            return self.searchedAlbums[1][self.searchedAlbums[0].index(_albumName)]
        except:pass
        self.searchedAlbums[0].append(_albumName)
        values, controlValue=[], 1
        while controlValue>0 and controlValue<4:
            try:
                returnedValues = Query().getReleases(ReleaseFilter(query=_albumName))
                controlValue=0
            except webservice.WebServiceError as errorDetails:
                if str(errorDetails)[:15]=="HTTP Error 503:":
                    time.sleep(controlValue)
                    controlValue+=1
                else:
                    raise ValueError(errorDetails)
        for result in returnedValues:
            values.append([result.score, result.release.id, result.release.title, result.release.asin,(result.release.textLanguage, "/", result.release.textScript), result.release.types, result.release.artist.id, result.release.artist.name])
        self.searchedAlbums[1].append(values)
        return values    

    def getSongsFromTitle(self, _titleName):
        """Returns titles that are similar to the one selected.
        Returned[x][0]:Match		    Returned[x][1]:Id              
        Returned[x][2]:Title		    Returned[x][3]:Length
        Returned[x][4]:Album Id		  Returned[x][5]:Album Name
        Returned[x][6]:Artist Id	  Returned[x][7]:Artist Name"""
        try:
            return self.searchedTitles[1][self.searchedTitles[0].index(_titleName)]
        except:pass
        self.searchedTitles[0].append(_titleName)
        values, controlValue=[], 1
        while controlValue>0 and controlValue<4:
            try:
                returnedValues = Query().getTracks(TrackFilter(query=_titleName))
                controlValue=0
            except webservice.WebServiceError as errorDetails:
                if str(errorDetails)[:15]=="HTTP Error 503:":
                    time.sleep(controlValue)
                    controlValue+=1
                else:
                    raise ValueError(errorDetails)
        for result in returnedValues:
            values.append([result.score, result.track.id, result.track.title, result.track.duration,result.track.releases[0].id ,result.track.releases[0].title ,result.track.artist.id ,result.track.artist.name ])
        self.searchedTitles[1].append(values)
        return values

    def getDetailsOfArtist(self, _artistId):
        """Returns all details for the selected artist ID.
        Returned[0]:Id			    Returned[1]:Name
        Returned[2]:Nickname		Returned[3]:Unique Name
        Returned[4]:Genre		    Returned[5]:Beginning Date
        Returned[6]:End Date		Returned[7]:Tags"""
        try:
            return self.searchedDetailsOfArtist[1][self.searchedDetailsOfArtist[0].index(_artistId)]
        except:pass
        self.searchedDetailsOfArtist[0].append(_artistId)
        controlValue=1
        while controlValue>0 and controlValue<4:
            try:
                q = Query()
                inc = webservice.ArtistIncludes(tags=True)
                artist = q.getArtistById(_artistId, inc)
                controlValue=0
            except webservice.WebServiceError as errorDetails:
                if str(errorDetails)[:15]=="HTTP Error 503:":
                    time.sleep(controlValue)
                    controlValue+=1
                else:
                    raise ValueError(errorDetails)
        values = [artist.id, artist.name, artist.sortName, artist.getUniqueName(), artist.type, artist.beginDate, artist.endDate, ", ".join(t.value for t in artist.tags)]
        self.searchedDetailsOfArtist[1].append(values)
        return values

    def getAlbumsOfArtist(self, _artistId, _searchDepth):
        """Returns all albums for the selected artist ID.
        Returned[x][0]:Id               Returned[x][1]:Title
        Returned[x][2]:Asin             Returned[x][3]:Text
        Returned[x][4]:Genre"""
        try:
            return self.searchedAlbumsOfArtist[1][self.searchedAlbumsOfArtist[0].index(_artistId)]
        except:pass
        self.searchedAlbumsOfArtist[0].append(_artistId)
        values=[]
        albumTypes = [model.Release.TYPE_ALBUM, model.Release.TYPE_SINGLE, model.Release.TYPE_COMPILATION, 
                        model.Release.TYPE_SOUNDTRACK, model.Release.TYPE_REMIX, model.Release.TYPE_SPOKENWORD, 
                        model.Release.TYPE_INTERVIEW, model.Release.TYPE_AUDIOBOOK, model.Release.TYPE_LIVE, 
                        model.Release.TYPE_EP, model.Release.TYPE_OTHER]
        albumStates = [model.Release.TYPE_OFFICIAL, model.Release.TYPE_PROMOTION, 
                          model.Release.TYPE_BOOTLEG, model.Release.TYPE_PSEUDO_RELEASE]
        for albumStateNo, albumState in enumerate(albumStates):
            if albumStateNo==0 or (albumStateNo==1 and _searchDepth>4) or (albumStateNo==2 and _searchDepth>7) or (albumStateNo==3 and _searchDepth>10):
                for albumTypeNo, albumType in enumerate(albumTypes):
                    if albumTypeNo<=_searchDepth:
                        controlValue = 1
                        while controlValue>0 and controlValue<4:
                            try:
                                q = Query()
                                inc = webservice.ArtistIncludes(releases=(albumState, albumType),tags=True)
                                artist = q.getArtistById(_artistId, inc)
                                controlValue=0
                            except webservice.WebServiceError as errorDetails:
                                if str(errorDetails)[:15]=="HTTP Error 503:":
                                    time.sleep(controlValue)
                                    controlValue+=1
                                else:
                                    raise ValueError(errorDetails)
                        for release in artist.getReleases():
                            values.append([release.id, release.title, release.asin, (release.textLanguage, "/", release.textScript), release.types])
        self.searchedAlbumsOfArtist[1].append(values)
        return values

    def getDetailsOfAlbum(self, _albumId):
        """Returns all details for the selected album ID.
        Returned[0]:Id              			  Returned[1]:Title
        Returned[2]:Asin	                	Returned[3]:Language
        Returned[4]:Artist ID	  	        	Returned[5]:Artist Name
        Returned[6]:Artist Nickname			    Returned[7]:First Release Date
        Returned[8]:Release Dates(By the countries)
        Returned[8][0]:Country				      Returned[8][1]:Release Date
        Returned[8][2]:Catalogue no			    Returned[8][3]:BarCode
        Returned[8][4]:Label name
        Returned[9]:CDs
        Returned[9][0]:CD ID				        Returned[9][1]:Disc Sector"""
        try:
            return self.searchedDetailsOfAlbum[1][self.searchedDetailsOfAlbum[0].index(_albumId)]
        except:pass
        self.searchedDetailsOfAlbum[0].append(_albumId)
        controlValue=1
        while controlValue>0 and controlValue<4:
            try:
                q = webservice.Query()
                inc = webservice.ReleaseIncludes(artist=True, releaseEvents=True, labels=True, discs=True, tracks=True)
                release = q.getReleaseById(_albumId, inc)
                controlValue=0
            except webservice.WebServiceError as errorDetails:
                if str(errorDetails)[:15]=="HTTP Error 503:":
                    time.sleep(controlValue)
                    controlValue+=1
                else:
                    raise ValueError(errorDetails)
        releaseDates=[]
        for event in release.releaseEvents:
            releaseDates.append([utils.getCountryName(event.country), event.date, 
                                ("#" + event.catalogNumber), 
                                ("EAN=" + event.barcode), 
                                ("(" + event.label.name + ")")])
        diskler=[]
        for disc in release.discs:
            diskler.append([disc.id, disc.sectors])
        values = [release.id, release.title, release.asin, (release.textLanguage, "/", release.textScript), 
                release.artist.id, release.artist.name, release.artist.sortName, 
                release.getEarliestReleaseDate(), releaseDates, diskler]
        self.searchedDetailsOfAlbum[1].append(values)
        return values

    def getSongsOfAlbum(self, _albumId):
        """Returns all titles for the selected album ID..
        Returned[x][0]:Id               Returned[x][1]:Title
        Returned[x][2]:Length"""
        try:
            return self.searchedSongsOfAlbum[1][self.searchedSongsOfAlbum[0].index(_albumId)]
        except:pass
        self.searchedSongsOfAlbum[0].append(_albumId)
        values, controlValue=[], 1
        while controlValue>0 and controlValue<4:
            try:
                q = webservice.Query()
                inc = webservice.ReleaseIncludes(artist=True, releaseEvents=True, labels=True, discs=True, tracks=True)
                release = q.getReleaseById(_albumId, inc)
                controlValue=0
            except webservice.WebServiceError as errorDetails:
                if str(errorDetails)[:15]=="HTTP Error 503:":
                    time.sleep(controlValue)
                    controlValue+=1
                else:
                    raise ValueError(errorDetails)
        for track in release.tracks:
            values.append([track.id, track.title, track.duration])
        self.searchedSongsOfAlbum[1].append(values)
        return values

    def getArtistIdFromArtist(self, _artistName):
        for artists in self.searchedArtists[1]:
            for artist in artists:
                if artist[2]==_artistName:
                    return artist[1]
        for albums in self.searchedAlbums[1]:
            for album in albums:
                if album[7]==_artistName:
                    return album[6]
        for titles in self.searchedTitles[1]:
            for title in titles:
                if title[7]==_artistName:
                    return title[6]
        return -1
        
    def getAlbumIdFromAlbum(self, _albumName):
        for albums in self.searchedAlbums[1]:
            for album in albums:
                if album[2]==_albumName:
                    return album[1]
        for titles in self.searchedTitles[1]:
            for title in titles:
                if title[5]==_albumName:
                    return title[4]
        for albums in self.searchedAlbumsOfArtist[1]:
            for album in albums:
                if album[1]==_albumName:
                    return album[0]
        return -1
        
    def getTitleIdFromTitle(self, _titleName):
        for titles in self.searchedTitles[1]:
            for title in titles:
                if title[2]==_titleName:
                    return title[1]
        for titles in self.searchedSongsOfAlbum[1]:
            for title in titles:
                if title[1]==_titleName:
                    return title[0]
        return -1
        
    def showInList(self):
        HBoxs=[]
        if len(self.trueSongs)>0 or len(self.falseSongs)>0 or len(self.songsOfAlbum)>0 or len(self.songsOfArtist)>0:
            tagNames = [translate("MusicTable", "Artist"), 
                        translate("MusicTable", "Title"), 
                        translate("MusicTable", "Album")]
            tagNamesKeys = ["Artist", "Title","Album", "File Name"]
            HBoxs.append(MHBoxLayout())
            HBoxs[-1].addWidget(MLabel(tagNames[0]))
            HBoxs[-1].addWidget(MLabel(tagNames[1]))
            HBoxs[-1].addWidget(MLabel(tagNames[2]))
            if len(self.trueSongs)>0:
                HBoxs.append(MHBoxLayout())
                HBoxs[-1].addWidget(MLabel(translate("SearchEngines", "Songs identified correctly:")))
                for song in self.trueSongs:
                    HBoxs.append(MHBoxLayout())
                    for no in range(len(tagNames)):
                        self.cbTags.append(MComboBox())
                        self.cbTags[-1].setObjectName(tagNamesKeys[no]+str(song[3]))
                        self.cbTags[-1].setEditable(True)
                        self.cbTags[-1].setMaximumWidth(200)
                        self.cbTags[-1].setMinimumWidth(200)
                        self.cbTags[-1].setToolTip(trForUI(song[no+4]))
                        for tag in song[no]:
                            self.cbTags[-1].addItem(trForUI(tag))
                        HBoxs[-1].addWidget(self.cbTags[-1])
            if len(self.falseSongs)>0:
                HBoxs.append(MHBoxLayout())
                HBoxs[-1].addWidget(MLabel(translate("SearchEngines", "Songs identified correctly but with errors:")))
                for song in self.falseSongs:
                    HBoxs.append(MHBoxLayout())
                    for no in range(len(tagNames)):
                        self.cbTags.append(MComboBox())
                        self.cbTags[-1].setObjectName(tagNamesKeys[no]+str(song[3]))
                        self.cbTags[-1].setEditable(True)
                        self.cbTags[-1].setMaximumWidth(200)
                        self.cbTags[-1].setMinimumWidth(200)
                        self.cbTags[-1].setToolTip(trForUI(song[no+4]))
                        for tag in song[no]:
                            self.cbTags[-1].addItem(trForUI(tag))
                        HBoxs[-1].addWidget(self.cbTags[-1])
            if len(self.songsOfAlbum)>0:
                HBoxs.append(MHBoxLayout())
                HBoxs[-1].addWidget(MLabel(translate("SearchEngines", "Songs searched with album name:")))
                for song in self.songsOfAlbum:
                    HBoxs.append(MHBoxLayout())
                    for no in range(len(tagNames)):
                        self.cbTags.append(MComboBox())
                        self.cbTags[-1].setObjectName(tagNamesKeys[no]+str(song[3]))
                        self.cbTags[-1].setEditable(True)
                        self.cbTags[-1].setMaximumWidth(200)
                        self.cbTags[-1].setMinimumWidth(200)
                        self.cbTags[-1].setToolTip(trForUI(song[no+4]))
                        HBoxs[-1].addWidget(self.cbTags[-1])
                        if no==0:
                            for tag in song[no]:
                                if self.cbTags[-1].findText(trForUI(tag))==-1:
                                    self.cbTags[-1].addItem(trForUI(tag))
                            self.cbTags[-1].addItem(translate("SearchEngines", "All Artists"))
                            MObject.connect(self.cbTags[-1],SIGNAL("currentIndexChanged(int)"),self.artistChanged)
                        elif no==1:
                            for tag in song[no]:
                                for t in tag:
                                    for bil in t:
                                        if self.cbTags[-1].findText(trForUI(bil))==-1:
                                            self.cbTags[-1].addItem(trForUI(bil))
                        elif no==2:
                            for tag in song[no]:
                                for t in tag:
                                    if self.cbTags[-1].findText(trForUI(t))==-1:
                                        self.cbTags[-1].addItem(trForUI(t))
                            self.cbTags[-1].addItem(translate("SearchEngines", "All Albums"))   
                            MObject.connect(self.cbTags[-1],SIGNAL("currentIndexChanged(int)"),self.albumChanged)
            if len(self.songsOfArtist)>0:
                HBoxs.append(MHBoxLayout())
                HBoxs[-1].addWidget(MLabel(translate("SearchEngines", "Songs searched with artist name:")))
                for song in self.songsOfArtist:
                    HBoxs.append(MHBoxLayout())
                    for no in range(len(tagNames)):
                        self.cbTags.append(MComboBox())
                        self.cbTags[-1].setObjectName(tagNamesKeys[no]+str(song[3]))
                        self.cbTags[-1].setEditable(True)
                        self.cbTags[-1].setMaximumWidth(200)
                        self.cbTags[-1].setMinimumWidth(200)
                        self.cbTags[-1].setToolTip(trForUI(song[no+4]))
                        HBoxs[-1].addWidget(self.cbTags[-1])
                        if no==0:
                            for tag in song[no]:
                                if self.cbTags[-1].findText(trForUI(tag))==-1:
                                    self.cbTags[-1].addItem(trForUI(tag))
                            self.cbTags[-1].addItem(translate("SearchEngines", "All Artists"))
                            MObject.connect(self.cbTags[-1],SIGNAL("currentIndexChanged(int)"),self.artistChanged)
                        elif no==1:
                            for tag in song[no]:
                                for t in tag:
                                    for s in t:
                                        if self.cbTags[-1].findText(trForUI(s))==-1:
                                            self.cbTags[-1].addItem(trForUI(s))
                        elif no==2:
                            for tag in song[no]:
                                for t in tag:
                                    if self.cbTags[-1].findText(trForUI(t))==-1:
                                        self.cbTags[-1].addItem(trForUI(t))
                            self.cbTags[-1].addItem(translate("SearchEngines", "All Albums"))    
                            MObject.connect(self.cbTags[-1],SIGNAL("currentIndexChanged(int)"),self.albumChanged)
        if len(self.nullSongs)>0:
            tagNames = [translate("MusicTable", "File Name"), 
                        translate("MusicTable", "Artist"), 
                        translate("MusicTable", "Title"), 
                        translate("MusicTable", "Album")]
            tagNamesKeys = ["File Name", "Artist", "Title", "Album"]
            HBoxs.append(MHBoxLayout())
            HBoxs[-1].addWidget(MLabel(tagNames[0]))
            HBoxs[-1].addWidget(MLabel(tagNames[1]))
            HBoxs[-1].addWidget(MLabel(tagNames[2]))
            HBoxs[-1].addWidget(MLabel(tagNames[3]))
            HBoxs.append(MHBoxLayout())
            HBoxs[-1].addWidget(MLabel(translate("SearchEngines", "Songs identified incorrectly:")))
            for song in self.nullSongs:
                HBoxs.append(MHBoxLayout())
                for no in range(len(tagNames)):
                    self.cbTags.append(MComboBox())
                    self.cbTags[-1].setObjectName(tagNamesKeys[no]+str(song[4]))
                    self.cbTags[-1].setEditable(True)
                    self.cbTags[-1].setMaximumWidth(150)
                    self.cbTags[-1].setMinimumWidth(150)
                    for tag in song[no]:
                        self.cbTags[-1].addItem(trForUI(tag))
                    HBoxs[-1].addWidget(self.cbTags[-1])
        if len(self.incorrectSongs)>0:
            tagNames= [tagNames[0], tagNames[1], tagNames[2], translate("MusicTable", "File Name")]
            HBoxs.append(MHBoxLayout())
            HBoxs[-1].addWidget(MLabel(tagNames[0]))
            HBoxs[-1].addWidget(MLabel(tagNames[1]))
            HBoxs[-1].addWidget(MLabel(tagNames[2]))
            HBoxs[-1].addWidget(MLabel(tagNames[3]))
            HBoxs.append(MHBoxLayout())
            HBoxs[-1].addWidget(MLabel(translate("SearchEngines", "Songs that caused errors:")))
            for song in self.incorrectSongs:
                HBoxs.append(MHBoxLayout())
                for no in range(len(tagNames)):
                    self.cbTags.append(MComboBox())
                    self.cbTags[-1].setObjectName(tagNamesKeys[no]+str(song[4]))
                    self.cbTags[-1].setEditable(True)
                    self.cbTags[-1].setMaximumWidth(150)
                    self.cbTags[-1].setMinimumWidth(150)
                    self.cbTags[-1].setToolTip(trForUI(song[no+4]))
                    for tag in song[no]:
                        self.cbTags[-1].addItem(trForUI(tag))
                    HBoxs[-1].addWidget(self.cbTags[-1])
        for box in HBoxs:
            self.vblPanel.addLayout(box)
        self.pnlPanel.setFixedSize(620,len(HBoxs)*30)
        self.checkSuggest()

    
    def checkSuggest(self):
        if len(self.searchedAlbums[0])==1 and len(self.rows)>1:
            self.sortBySelectedAlbum()
        if len(self.searchedArtists[0])==1 and len(self.rows)>1:
            self.showSuggest()
    
    def selectAlbum(self):
        for rowNo in self.rows:
            if len(self.rows)-1!=self.rows[-1]:
                rowNo=rowNo/2
            self.findChild(MComboBox, "Album"+str(rowNo)).setCurrentIndex(self.findChild(MComboBox, "Album0").currentIndex())
    
    def sortBySelectedAlbum(self):
        for rowNo in self.rows:
            if len(self.rows)-1!=self.rows[-1]:
                rowNo=rowNo/2
            self.findChild(MComboBox, "Title"+str(rowNo)).setCurrentIndex(rowNo)

    def showSuggest(self, _isHidden=False):
        self.lblSuggest = MLabel(self)
        self.pbtnSuggest = MPushButton("", self)
        self.pbtnSuggest.setFlat(True)
        self.movie = MMovie("Images:suggest.gif")
        self.lblSuggest.setMovie(self.movie)
        self.movie.start()
        if _isHidden:
            self.pbtnSuggest.setVisible(False)
            self.lblSuggest.setVisible(False)
        else:
            self.lblSuggest.setGeometry(600, 25, 30, 30)
            self.pbtnSuggest.setGeometry(600, 30, 20, 20)
            self.mSuggest = MMenu(self)
            self.mSuggest.clear()
            self.labelsOfSuggests = [translate("SearchEngines", "Sort Titles By Album")]
            for label in self.labelsOfSuggests:
                action = MAction(label, self.mSuggest)
                action.setObjectName(label)
                self.mSuggest.addAction(action)
            MObject.connect(self.mSuggest,SIGNAL("triggered(QAction *)"),self.applySuggest)
            self.mSuggest.setGeometry(self.pbtnSuggest.x()+10,self.pbtnSuggest.y()+10,
                                       self.mSuggest.width(),self.mSuggest.height())
            self.pbtnSuggest.setMenu(self.mSuggest)
            self.lblSuggest.setVisible(True)
            self.pbtnSuggest.setVisible(True)
            
    def applySuggest(self, _action):
        if str(_action.objectName()) == str(self.labelsOfSuggests[0]):
            isPracticable = True
            id= self.getAlbumIdFromAlbum(self.findChild(MComboBox, "Album0").currentText())
            if len(self.rows)!=len(self.getSongsOfAlbum(id)):
                isPracticable = False
                answer = Dialogs.ask(translate("SearchEngines", "Number Of Songs Are Different"), 
                                translate("SearchEngines", "The number of songs for the album you selected is not the same with the album you have verified.<br>Do you want to sort by the album anyway?"))
                if answer==Dialogs.Yes:
                    isPracticable = True
            if isPracticable:
                self.selectAlbum()
                self.sortBySelectedAlbum()
            
    def apply(self):
        self.parent().createHistoryPoint()
        songs=[]
        for tag in self.trueSongs:
            artist = str(self.findChild(MComboBox, trForUI("Artist"+str(tag[3]))).currentText())
            title = str(self.findChild(MComboBox, trForUI("Title"+str(tag[3]))).currentText())
            album = str(self.findChild(MComboBox, trForUI("Album"+str(tag[3]))).currentText())
            songs.append([artist,title,album,tag[3]])   
        for tag in self.falseSongs:
            artist = str(self.findChild(MComboBox, trForUI("Artist"+str(tag[3]))).currentText())
            title = str(self.findChild(MComboBox, trForUI("Title"+str(tag[3]))).currentText())
            album = str(self.findChild(MComboBox, trForUI("Album"+str(tag[3]))).currentText())
            songs.append([artist,title,album,tag[3]])
        for tag in self.songsOfAlbum:
            artist = str(self.findChild(MComboBox, trForUI("Artist"+str(tag[3]))).currentText())
            title = str(self.findChild(MComboBox, trForUI("Title"+str(tag[3]))).currentText())
            album = str(self.findChild(MComboBox, trForUI("Album"+str(tag[3]))).currentText())
            songs.append([artist,title,album,tag[3]])
        for tag in self.songsOfArtist:
            artist = str(self.findChild(MComboBox, trForUI("Artist"+str(tag[3]))).currentText())
            title = str(self.findChild(MComboBox, trForUI("Title"+str(tag[3]))).currentText())
            album = str(self.findChild(MComboBox, trForUI("Album"+str(tag[3]))).currentText())
            songs.append([artist,title,album,tag[3]])
        for tag in self.nullSongs:
            dosya = str(self.findChild(MComboBox, trForUI("File Name"+str(tag[4]))).currentText())
            artist = str(self.findChild(MComboBox, trForUI("Artist"+str(tag[4]))).currentText())
            title = str(self.findChild(MComboBox, trForUI("Title"+str(tag[4]))).currentText())
            album = str(self.findChild(MComboBox, trForUI("Album"+str(tag[4]))).currentText())
            songs.append([dosya,artist,title,album,tag[4]])
        for tag in self.incorrectSongs:
            dosya = str(self.findChild(MComboBox, trForUI("File Name"+str(tag[4]))).currentText())
            artist = str(self.findChild(MComboBox, trForUI("Artist"+str(tag[4]))).currentText())
            title = str(self.findChild(MComboBox, trForUI("Title"+str(tag[4]))).currentText())
            album = str(self.findChild(MComboBox, trForUI("Album"+str(tag[4]))).currentText())
            songs.append([dosya,artist,title,album,tag[4]])
        for song in songs:
            if len(song)==4:
                if str(self.parent().item(song[3],2).text())!=song[0]:
                    self.parent().setItem(song[3],2,MTableWidgetItem(trForUI(song[0])))
                if str(self.parent().item(song[3],3).text())!=song[1]:
                    self.parent().setItem(song[3],3,MTableWidgetItem(trForUI(song[1])))
                if str(self.parent().item(song[3],4).text())!=song[2]:
                    self.parent().setItem(song[3],4,MTableWidgetItem(trForUI(song[2])))
            else:
                if str(self.parent().item(song[4],1).text())!=song[0]:
                    self.parent().setItem(song[4],2,MTableWidgetItem(trForUI(song[0])))
                if str(self.parent().item(song[4],2).text())!=song[1]:
                    self.parent().setItem(song[4],2,MTableWidgetItem(trForUI(song[1])))
                if str(self.parent().item(song[4],3).text())!=song[2]:
                    self.parent().setItem(song[4],3,MTableWidgetItem(trForUI(song[2])))
                if str(self.parent().item(song[4],4).text())!=song[3]:
                    self.parent().setItem(song[4],4,MTableWidgetItem(trForUI(song[3])))
        self.close()

    def artistChanged(self, _index):
        if self.isAlterArtist:
            if _index == self.sender().currentIndex():
                self.isArtistImportant = False
            else:
                self.isArtistImportant = True
            self.isArtistChanged = True
            rowNo=int(str(self.sender().objectName()).replace("Artist",""))
            self.isAlterAlbum=True
            cbTag = self.sender()
            for object in self.cbTags:
                if str(object.objectName())=="Title"+str(rowNo):
                    object.clear()
                    for song in self.songsOfArtist:
                        if song[3]==rowNo:
                            for x,tag in enumerate(song[1]):
                                if _index!=self.sender().count()-1:
                                    if song[0][x]==str(cbTag.currentText()):
                                        for t in tag:
                                            for s in t:
                                                if object.findText(trForUI(s))==-1:
                                                    object.addItem(trForUI(s))
                                else:
                                    for t in tag:
                                        for s in t:
                                            if object.findText(trForUI(s))==-1:
                                                object.addItem(trForUI(s))
                            break
                    for song in self.songsOfAlbum:
                        if song[3]==rowNo:
                            for x,tag in enumerate(song[1]):
                                if _index!=self.sender().count()-1:
                                    if song[0][x]==str(cbTag.currentText()):
                                        for t in tag:
                                            for s in t:
                                                if object.findText(trForUI(s))==-1:
                                                    object.addItem(trForUI(s))
                                else:
                                    for t in tag:
                                        for s in t:
                                            if object.findText(trForUI(s))==-1:
                                                object.addItem(trForUI(s))
                            break
                if str(object.objectName())=="Album"+str(rowNo):
                    object.clear()
                    for song in self.songsOfArtist:
                        if song[3]==rowNo:
                            for x,tag in enumerate(song[2]):
                                if _index!=cbTag.count()-1:
                                    if song[0][x]==str(cbTag.currentText()):
                                        for t in tag:
                                            if object.findText(trForUI(t))==-1:
                                                object.addItem(trForUI(t))
                                else:
                                    for t in tag:
                                        if object.findText(trForUI(t))==-1:
                                            object.addItem(trForUI(t))
                            object.addItem(translate("SearchEngines", "All Albums"))
                            break
                    for song in self.songsOfAlbum:
                        if song[3]==rowNo:
                            for x,tag in enumerate(song[2]):
                                if _index!=cbTag.count()-1:
                                    if song[0][x]==str(cbTag.currentText()):
                                        for t in tag:
                                            if object.findText(trForUI(t))==-1:
                                                object.addItem(trForUI(t))
                                else:
                                    for t in tag:
                                        if object.findText(trForUI(t))==-1:
                                            object.addItem(trForUI(t))
                            object.addItem(translate("SearchEngines", "All Albums"))
                            break
            self.isAlterAlbum=False

    def albumChanged(self, _index):
        if _index == self.sender().count()-1:
            self.isArtistChangedTemp = self.isArtistChanged
            self.isArtistImportantTemp = self.isArtistImportant
            self.isArtistChanged = True
            self.isArtistImportant = True
        if self.isAlterAlbum==False:
            cbTag = self.sender()
            rowNo=int(str(cbTag.objectName()).replace("Album",""))
            for object in self.cbTags:
                if str(object.objectName())=="Artist"+str(rowNo):
                    artistObject=object
                    break
            for object in self.cbTags:
                if str(object.objectName())=="Title"+str(rowNo):
                    object.clear()
                    for song in self.songsOfArtist:
                        if song[3]==rowNo:
                            if self.isArtistImportant==True and self.isArtistChanged==True:
                                if artistObject.currentIndex() != artistObject.count()-1:
                                    if _index!=cbTag.count()-1:
                                        for t in song[1][artistObject.currentIndex()][cbTag.currentIndex()]:
                                            if object.findText(trForUI(t))==-1:
                                                object.addItem(trForUI(t))
                                    else:
                                        for y, tag in enumerate(song[2][artistObject.currentIndex()]):
                                            for t in song[1][artistObject.currentIndex()][y]:
                                                if object.findText(trForUI(t))==-1:
                                                    object.addItem(trForUI(t))
                                else:
                                    if _index!=cbTag.count()-1:
                                        for x, tag in enumerate(song[0]):
                                            for y, tag1 in enumerate(song[2][x]): 
                                                if song[2][x][y]==cbTag.currentText():
                                                    for t in song[1][x][y]:
                                                        if object.findText(trForUI(t))==-1:
                                                            object.addItem(trForUI(t))
                                    else:
                                        for x, tag in enumerate(song[0]):
                                            for y, tag1 in enumerate(song[2][x]):
                                                for t in song[1][x][y]:
                                                    if object.findText(trForUI(t))==-1:
                                                        object.addItem(trForUI(t))
                            else:
                                for x, tag in enumerate(song[1]):
                                    for y, tagm in enumerate(tag):
                                        if song[2][x][y] == cbTag.currentText():
                                            self.isAlterArtist = False
                                            artistObject.setCurrentIndex(x)
                                            self.isAlterArtist = True
                                            for t in song[1][x][y]:
                                                if object.findText(trForUI(t))==-1:
                                                    object.addItem(trForUI(t))
                    for song in self.songsOfAlbum:
                        if song[3]==rowNo:
                            if self.isArtistImportant==True and self.isArtistChanged==True:
                                if artistObject.currentIndex() != artistObject.count()-1:
                                    if _index!=cbTag.count()-1:
                                        for t in song[1][artistObject.currentIndex()][cbTag.currentIndex()]:
                                            if object.findText(trForUI(t))==-1:
                                                object.addItem(trForUI(t))
                                    else:
                                        for y, tag in enumerate(song[2][artistObject.currentIndex()]):
                                            for t in song[1][artistObject.currentIndex()][y]:
                                                if object.findText(trForUI(t))==-1:
                                                    object.addItem(trForUI(t))
                                else:
                                    if _index!=cbTag.count()-1:
                                        for x, tag in enumerate(song[0]):
                                            for y, tag1 in enumerate(song[2][x]): 
                                                if song[2][x][y]==cbTag.currentText():
                                                    for t in song[1][x][y]:
                                                        if object.findText(trForUI(t))==-1:
                                                            object.addItem(trForUI(t))
                                    else:
                                        for x, tag in enumerate(song[0]):
                                            for y, tag1 in enumerate(song[2][x]):
                                                for t in song[1][x][y]:
                                                    if object.findText(trForUI(t))==-1:
                                                        object.addItem(trForUI(t))
                            else:
                                for x, tag in enumerate(song[1]):
                                    for y, tagm in enumerate(tag):
                                        if song[2][x][y] == cbTag.currentText():
                                            self.isAlterArtist = False
                                            artistObject.setCurrentIndex(x)
                                            self.isAlterArtist = True
                                            for t in song[1][x][y]:
                                                if object.findText(trForUI(t))==-1:
                                                    object.addItem(trForUI(t))
                    break
        if _index == self.sender().count()-1:
            self.isArtistChanged = self.isArtistChangedTemp
            self.isArtistImportant = self.isArtistImportantTemp
            
            
            



## This file is part of HamsiManager.
## 
## Copyright (c) 2010 Murat Demir <mopened@gmail.com>      
##
## Hamsi Manager is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## Hamsi Manager is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with HamsiManager; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import Variables
import Universals

MStringList = None

if Variables.MyObjectName=="PyQt4" or Variables.MyObjectName=="":
    from PyQt4 import QtGui
    for obj in dir(QtGui):
        if obj[0]=="Q":
            exec ("M"+obj[1:]+" = QtGui." + obj)
        else:
            exec (obj + " = QtGui." + obj)
    from PyQt4 import QtCore
    for obj in dir(QtCore):
        if obj[0]=="Q":
            exec ("M"+obj[1:]+" = QtCore." + obj)
        else:
            exec (obj + " = QtCore." + obj)
    from PyQt4 import QtNetwork
    for obj in dir(QtNetwork):
        if obj[0]=="Q":
            exec ("M"+obj[1:]+" = QtNetwork." + obj)
        else:
            exec (obj + " = QtNetwork." + obj)
elif Variables.MyObjectName=="PySide":
    from PySide import QtGui
    for obj in dir(QtGui):
        if obj[0]=="Q":
            exec ("M"+obj[1:]+" = QtGui." + obj)
        else:
            exec (obj + " = QtGui." + obj)
    from PySide import QtCore
    for obj in dir(QtCore):
        if obj[0]=="Q":
            exec ("M"+obj[1:]+" = QtCore." + obj)
        else:
            exec (obj + " = QtCore." + obj)
    from PySide import QtNetwork
    for obj in dir(QtNetwork):
        if obj[0]=="Q":
            exec ("M"+obj[1:]+" = QtNetwork." + obj)
        else:
            exec (obj + " = QtNetwork." + obj)
            
if Variables.MyObjectName=="PyQt4" and "isActivePyKDE4" in Universals.MySettings.keys():
    if Universals.isActivePyKDE4==True:
        try:
            from PyKDE4 import kdeui
            for obj in dir(kdeui):
                if obj[0]=="K":
                    exec ("M"+obj[1:]+" = kdeui." + obj)
                else:
                    exec (obj + " = kdeui." + obj)
            from PyKDE4 import kdecore
            for obj in dir(kdecore):
                if obj[0]=="K":
                    exec ("M"+obj[1:]+" = kdecore." + obj)
                else:
                    exec (obj + " = kdecore." + obj)
            from PyKDE4 import kio
            for obj in dir(kio):
                if obj[0]=="K":
                    exec ("M"+obj[1:]+" = kio." + obj)
                else:
                    exec (obj + " = kio." + obj)
            #this PyKDE4 objects different from PyQt4 objects
            QMessageBox = QtGui.QMessageBox
            QDirModel = QtGui.QDirModel
            QIcon = QtGui.QIcon
            MFileDialog = QtGui.QFileDialog 
            MLocale = QtCore.QLocale
            KFileDialog = kio.KFileDialog
            KLocale = kdecore.KLocale
        except:
            Universals.isActivePyKDE4 = False
else:
    #PySide not using with PyKDE4
    Universals.isActivePyKDE4 = False
    
if MStringList is None:
    def MStringList(_s):
        return [_s]
    
def translate(_p, _s):
    return str(MApplication.translate(_p, _s))
    
def trForUI(_s):
    _s = str(_s)
    return _s
    
def trForM(_s):
    _s = str(_s)
    return _s
    
def getMyObject(_objectName):
    MyObject = None
    if Variables.MyObjectName=="PySide":
        MyObject = __import__("PySide." + _objectName, globals(), locals(), [_objectName], -1)
    elif Variables.MyObjectName=="PyQt4" or Variables.MyObjectName=="":
        MyObject = __import__("PyQt4." + _objectName, globals(), locals(), [_objectName], -1)
    return MyObject
            
def getMyDialog():
    try:
        if Universals.MainWindow.objectName()=="RealMainWindow":
            return MDialog, "MDialog", Universals.MainWindow
        else:
            return MMainWindow, "MMainWindow", None
    except:
        return MMainWindow, "MMainWindow", None
        
def moveToCenter(_dialog):
    scrn = 0
    if MApplication.desktop().isVirtualDesktop():
        scrn = MApplication.desktop().screenNumber(MCursor.pos())
    else:
        scrn = MApplication.desktop().screenNumber(_dialog)
    desk = MApplication.desktop().availableGeometry(scrn)
    _dialog.move(int((desk.width() - _dialog.width()) / 2),
      int((desk.height() - _dialog.height()) / 2))
    
def setCompleter(_object, _objectName=None):
    if Universals.getBoolValue("isShowAllForCompleter"):
        _objectName = "%*%"
    from Databases import CompleterTable
    if _objectName==None:
        _objectName = _object.objectName()
    _objectName = str(_objectName)
    cmpCompleter = MCompleter(CompleterTable.fetchAllByObjectName(_objectName))
    cmpCompleter.setCaseSensitivity(Mt.CaseInsensitive)
    _object.setCompleter(cmpCompleter)

Universals.isLoadedMyObjects = True



## This file is part of HamsiManager.
## 
## Copyright (c) 2010 Murat Demir <mopened@gmail.com>      
##
## Hamsi Manager is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## Hamsi Manager is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with HamsiManager; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import Variables
from MyObjects import *
import Universals
import Dialogs
import Settings
import InputOutputs
import sys
import MyConfigure
import ReportBug

class MyPlugins(MDialog):
    global installPlugin
    
    def __init__(self, _parent):
        MDialog.__init__(self, _parent)
        if Universals.isActivePyKDE4==True:
            self.setButtons(MDialog.NoDefault)
        self.lstwPluginList = MListWidget()
        self.pbtnInstall = MPushButton(translate("MyPlugins", "Install The Selected Plug-in"))
        pbtnClose = MPushButton(translate("MyPlugins", "Close"))
        self.fillPlugins()
        self.connect(self.pbtnInstall,SIGNAL("clicked()"),self.installThis)
        self.connect(pbtnClose,SIGNAL("clicked()"),self.close)
        pnlMain = MWidget(self)
        vblMain = MVBoxLayout(pnlMain)
        HBox1 = MHBoxLayout()
        HBox1.addWidget(self.pbtnInstall)
        HBox1.addWidget(pbtnClose)
        vblMain.addWidget(self.lstwPluginList)
        vblMain.addStretch(1)
        vblMain.addLayout(HBox1)
        if Universals.isActivePyKDE4==True:
            self.setMainWidget(pnlMain)
        else:
            self.setLayout(vblMain)
        self.setWindowTitle(translate("MyPlugins", "My Plug-ins"))
        self.show()
    
    def fillPlugins(self):
        self.lstwPluginList.clear()
        self.myPluginsNames = Variables.getMyPluginsNames()
        for plugin in self.myPluginsNames:
            pluginModule = __import__("MyPlugins." + plugin, globals(), locals(), ["pluginName", "pluginVersion", "isInstallable"], -1)
            if pluginModule.isInstallable():
                installedVersion = Settings.getUniversalSetting(trForM(pluginModule.pluginName), "")
                if installedVersion == "":
                    details = translate("MyPlugins", "Could Not Be Determined")
                elif installedVersion != pluginModule.pluginVersion:
                    details = translate("MyPlugins", "Have A New Version")
                else:
                    details = translate("MyPlugins", "Installed")
                self.lstwPluginList.addItem(trForUI(pluginModule.pluginName) + "\n\t" + details)
        if self.lstwPluginList.count()==0:
            self.lstwPluginList.addItem(translate("MyPlugins", "Could not find the appropriate plug-in to your system"))
            self.pbtnInstall.setEnabled(False)
    
    def installThis(self):
        try:
            installPlugin(self.myPluginsNames[self.lstwPluginList.currentRow()])
            self.fillPlugins()
        except:
            error = ReportBug.ReportBug()
            error.show()
    
    def installPlugin(_pluginName, _isQuiet=False):
        isInstalled = False
        pluginModule = __import__("MyPlugins." + _pluginName, globals(), locals(), ["pluginName", "pluginFiles", "pluginDirectory", "installThisPlugin", "setupDirectory", "pluginVersion"], -1)
        if installThisPlugin==None:
            if pluginModule.pluginDirectory=="":
                try:InputOutputs.IA.makeDirs(pluginModule.setupDirectory)
                except:pass
                for pluginFile in pluginModule.pluginFiles:
                    InputOutputs.IA.copyOrChange(Variables.HamsiManagerDirectory+"/MyPlugins/"+_pluginName+"/"+pluginFile, pluginModule.setupDirectory+"/"+pluginFile, "file", "only", True)
                    MyConfigure.reConfigureFile(pluginModule.setupDirectory+"/"+pluginFile, Variables.HamsiManagerDirectory)
                isInstalled = True
            else:
                oldFilePath = Variables.HamsiManagerDirectory+"/MyPlugins/"+_pluginName+"/"+pluginModule.pluginDirectory
                newFilePath = InputOutputs.IA.copyOrChange(oldFilePath, pluginModule.setupDirectory+"/"+pluginModule.pluginDirectory, "directory", "only", True)
                if newFilePath!=oldFilePath:
                    isInstalled = True
        else:
            isInstalled = pluginModule.installThisPlugin()
        if isInstalled:
            Settings.setUniversalSetting(trForM(pluginModule.pluginName), str(pluginModule.pluginVersion))
            if _isQuiet==False:
                Dialogs.show(translate("MyPlugins", "Plug-in Installation Is Complete"), 
                         str(translate("MyPlugins", "\"%s\" is installed on your system.")) % (pluginModule.pluginName))
        elif isInstalled=="AlreadyInstalled":
            if _isQuiet==False:
                Dialogs.show(translate("MyPlugins", "Plug-in Already Installed"), 
                         str(translate("MyPlugins", "\"%s\" already installed on your system.")) % (pluginModule.pluginName))
        else:
            if _isQuiet==False:
                Dialogs.showError(translate("MyPlugins", "Plug-in Installation Failed"), 
                         str(translate("MyPlugins", "\"%s\" failed to install on your system.")) % (pluginModule.pluginName))

 

class MyPluginsForSystem(MWidget):
    global installPlugin
    
    def __init__(self, _parent):
        MWidget.__init__(self, _parent)
        lblHeader = MLabel(trForUI("<b>" + translate("MyPlugins", "My Plug-ins") + "</b>"))
        lblNote = MLabel(translate("MyPlugins", "You can manage plugins in your system"))
        self.lstwPluginList = MListWidget()
        self.pbtnInstall = MPushButton(translate("MyPlugins", "Install The Selected Plug-in"))
        self.fillPlugins()
        self.connect(self.pbtnInstall,SIGNAL("clicked()"),self.installThis)
        pnlMain = MWidget(self)
        vblMain = MVBoxLayout(pnlMain)
        HBox1 = MHBoxLayout()
        HBox1.addStretch(10)
        HBox1.addWidget(self.pbtnInstall)
        vblMain.addWidget(lblHeader)
        vblMain.addWidget(lblNote)
        vblMain.addWidget(self.lstwPluginList)
        vblMain.addStretch(1)
        vblMain.addLayout(HBox1)
        self.setLayout(vblMain)
    
    def fillPlugins(self):
        self.lstwPluginList.clear()
        self.myPluginsNames = Variables.getMyPluginsNames()
        for plugin in self.myPluginsNames:
            pluginModule = __import__("MyPlugins." + plugin, globals(), locals(), ["pluginName", "pluginVersion", "isInstallable"], -1)
            if pluginModule.isInstallable():
                installedVersion = Settings.getUniversalSetting(trForM(pluginModule.pluginName), "")
                if installedVersion == "":
                    details = translate("MyPlugins", "Could Not Be Determined")
                elif installedVersion != pluginModule.pluginVersion:
                    details = translate("MyPlugins", "Have A New Version")
                else:
                    details = translate("MyPlugins", "Installed")
                self.lstwPluginList.addItem(trForUI(pluginModule.pluginName) + "\n\t" + details)
        if self.lstwPluginList.count()==0:
            self.lstwPluginList.addItem(translate("MyPlugins", "Could not find the appropriate plug-in to your system"))
            self.pbtnInstall.setEnabled(False)
    
    def installThis(self):
        try:
            installPlugin(self.myPluginsNames[self.lstwPluginList.currentRow()])
            self.fillPlugins()
        except:
            error = ReportBug.ReportBug()
            error.show()
    
    def installPlugin(_pluginName, _isQuiet=False):
        isInstalled = False
        pluginModule = __import__("MyPlugins." + _pluginName, globals(), locals(), ["pluginName", "pluginFiles", "pluginDirectory", "installThisPlugin", "setupDirectory", "pluginVersion"], -1)
        if pluginModule.installThisPlugin==None:
            if pluginModule.pluginDirectory=="":
                try:InputOutputs.IA.makeDirs(pluginModule.setupDirectory)
                except:pass
                for pluginFile in pluginModule.pluginFiles:
                    InputOutputs.IA.copyOrChange(Variables.HamsiManagerDirectory+"/MyPlugins/"+_pluginName+"/"+pluginFile, pluginModule.setupDirectory+"/"+pluginFile, "file", "only", True)
                    MyConfigure.reConfigureFile(pluginModule.setupDirectory+"/"+pluginFile, Variables.HamsiManagerDirectory)
                isInstalled = True
            else:
                oldFilePath = Variables.HamsiManagerDirectory+"/MyPlugins/"+_pluginName+"/"+pluginModule.pluginDirectory
                newFilePath = InputOutputs.IA.copyOrChange(oldFilePath, pluginModule.setupDirectory+"/"+pluginModule.pluginDirectory, "directory", "only", True)
                if newFilePath!=oldFilePath:
                    isInstalled = True
        else:
            isInstalled = pluginModule.installThisPlugin()
        if isInstalled:
            Settings.setUniversalSetting(trForM(pluginModule.pluginName), str(pluginModule.pluginVersion))
            if _isQuiet==False:
                Dialogs.show(translate("MyPlugins", "Plug-in Installation Is Complete"), 
                         str(translate("MyPlugins", "\"%s\" is installed on your system.")) % (pluginModule.pluginName))
        elif isInstalled=="AlreadyInstalled":
            if _isQuiet==False:
                Dialogs.show(translate("MyPlugins", "Plug-in Already Installed"), 
                         str(translate("MyPlugins", "\"%s\" already installed on your system.")) % (pluginModule.pluginName))
        else:
            if _isQuiet==False:
                Dialogs.showError(translate("MyPlugins", "Plug-in Installation Failed"), 
                         str(translate("MyPlugins", "\"%s\" failed to install on your system.")) % (pluginModule.pluginName))

 



## This file is part of HamsiManager.
## 
## Copyright (c) 2010 Murat Demir <mopened@gmail.com>      
##
## Hamsi Manager is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## Hamsi Manager is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with HamsiManager; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import Variables
import Universals
from Databases import sqlite, getDefaultConnection, correctForSql, getAmendedSQLInsertOrUpdateQueries

class BookmarksOfSpecialTools:
    global fetchAll, fetchAllByType, fetch, checkValues, insert, update, delete
    global getTableCreateQuery, getDeleteTableQuery, getDefaultsQueries
    global tableName, tableVersion, allForFetch, allForFetchByType
    tableName = "bookmarksOfSpecialTools"
    tableVersion = 2
    allForFetch, allForFetchByType = None, {}
        
    def fetchAll():
        global allForFetch
        if allForFetch==None:
            con = getDefaultConnection()
            cur = con.cursor()
            cur.execute("SELECT * FROM " + tableName)
            allForFetch = cur.fetchall()
        return allForFetch
        
    def fetchAllByType(_type=None):
        global allForFetchByType
        if _type==None:
            _type = Universals.MainWindow.Table.SubTable.specialTollsBookmarkPointer
        if _type not in allForFetchByType or allForFetchByType[_type]==None:
            import Organizer
            con = getDefaultConnection()
            cur = con.cursor()
            cur.execute("SELECT * FROM " + tableName + " where type='" + _type + "'")
            myBookmarks = []
            for mybm in cur.fetchall():
                tempT = mybm[2]
                tempString = tempT.split(";")
                tempT = ""
                for t in tempString[:-2]:
                    tempT+=t
                newText  = Organizer.whatDoesSpecialCommandDo("-",
                                tempString[-2],
                                tempT, False, True)
                myBookmarks.append([mybm[0], newText, mybm[2], mybm[3]])
            allForFetchByType[_type] = myBookmarks
        return allForFetchByType[_type]
    
    def fetch(_id):
        con = getDefaultConnection()
        cur = con.cursor()
        cur.execute("SELECT * FROM " + tableName + " where id=" + str(int(_id)))
        return cur.fetchall()
    
    def checkValues(_bookmark, _value, _type):
        if len(_bookmark)==0 or len(_value)==0:
            return False
        return True
    
    def insert(_bookmark, _value, _type=None):
        global allForFetch, allForFetchByType
        if _type==None:
            _type = Universals.MainWindow.Table.SubTable.specialTollsBookmarkPointer
        if checkValues(_bookmark, _value, _type):
            allForFetch, allForFetchByType[_type] = None, None
            con = getDefaultConnection()
            cur = con.cursor()
            sqlQueries = getAmendedSQLInsertOrUpdateQueries(tableName, {"bookmark" : "'" + correctForSql(_bookmark) + "'", "value" : "'" + correctForSql(_value) + "'", "type" : "'" + correctForSql(_type) + "'"}, ["value"])
            cur.execute(sqlQueries[0])
            cur.execute(sqlQueries[1])
            con.commit()
            cur.execute("SELECT last_insert_rowid();")
            return cur.fetchall()[0][0]
        return None
    
    def update(_id, _bookmark, _value, _type=None):
        global allForFetch, allForFetchByType
        if _type==None:
            _type = Universals.MainWindow.Table.SubTable.specialTollsBookmarkPointer
        if checkValues(_bookmark, _value, _type):
            allForFetch, allForFetchByType[_type] = None, None
            con = getDefaultConnection()
            cur = con.cursor()
            cur.execute(str("update " + tableName + " set bookmark='" + correctForSql(_bookmark) + "', value='" + correctForSql(_value) + "', type='" + correctForSql(_type) + "' where id=" + str(int(_id))))
            con.commit()
    
    def delete(_id, _type=None):
        if _type==None:
            _type = Universals.MainWindow.Table.SubTable.specialTollsBookmarkPointer
        global allForFetch, allForFetchByType
        allForFetch, allForFetchByType[_type] = None, None
        con = getDefaultConnection()
        cur = con.cursor()
        cur.execute("delete from " + tableName + " where id="+str(int(_id)))
        con.commit()
        
    def getTableCreateQuery():
        return "CREATE TABLE IF NOT EXISTS " + tableName + " ('id' INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,'bookmark' TEXT,'value' TEXT,'type' TEXT)"
        
    def getDeleteTableQuery():
        return "DELETE FROM " + tableName
        
    def getDefaultsQueries():
        sqlQueries = []
        sqlQueries += getAmendedSQLInsertOrUpdateQueries(tableName, {"bookmark" : "''", "value" : "'File Name , Artist - Title ;right;113'", "type" : "'music'"}, ["value"])
        sqlQueries += getAmendedSQLInsertOrUpdateQueries(tableName, {"bookmark" : "''", "value" : "'Artist - Title , File Name  ;left;113'", "type" : "'music'"}, ["value"])
        sqlQueries += getAmendedSQLInsertOrUpdateQueries(tableName, {"bookmark" : "''", "value" : "'Track No - Title , File Name  ;left;113'", "type" : "'music'"}, ["value"])
        sqlQueries += getAmendedSQLInsertOrUpdateQueries(tableName, {"bookmark" : "''", "value" : "'Artist - Album , Directory  ;left;113'", "type" : "'music'"}, ["value"])
        sqlQueries += getAmendedSQLInsertOrUpdateQueries(tableName, {"bookmark" : "''", "value" : "'File Name , Title  ;right;102'", "type" : "'music'"}, ["value"])
        sqlQueries += getAmendedSQLInsertOrUpdateQueries(tableName, {"bookmark" : "''", "value" : "'Title , File Name  ;right;102'", "type" : "'music'"}, ["value"])
        sqlQueries += getAmendedSQLInsertOrUpdateQueries(tableName, {"bookmark" : "''", "value" : "'Year , Album  ;right;102'", "type" : "'music'"}, ["value"])
        sqlQueries += getAmendedSQLInsertOrUpdateQueries(tableName, {"bookmark" : "''", "value" : "'Lyrics , Artist - Title  ;right;113'", "type" : "'music'"}, ["value"])
        sqlQueries += getAmendedSQLInsertOrUpdateQueries(tableName, {"bookmark" : "''", "value" : "'Artist - Album - Title , File Name  ;left;124'", "type" : "'music'"}, ["value"])
        sqlQueries += getAmendedSQLInsertOrUpdateQueries(tableName, {"bookmark" : "''", "value" : "'Directory - File Name , Directory  ;left;113'", "type" : "'file'"}, ["value"])
        sqlQueries += getAmendedSQLInsertOrUpdateQueries(tableName, {"bookmark" : "''", "value" : "'Directory , File Name  ;right;102'", "type" : "'file'"}, ["value"])
        sqlQueries += getAmendedSQLInsertOrUpdateQueries(tableName, {"bookmark" : "''", "value" : "'File Name , Directory  ;right;102'", "type" : "'file'"}, ["value"])
        sqlQueries += getAmendedSQLInsertOrUpdateQueries(tableName, {"bookmark" : "''", "value" : "'Directory , File/Directory Name  ;right;102'", "type" : "'directory'"}, ["value"])
        sqlQueries += getAmendedSQLInsertOrUpdateQueries(tableName, {"bookmark" : "''", "value" : "'File/Directory Name , Directory  ;right;102'", "type" : "'directory'"}, ["value"])
        sqlQueries += getAmendedSQLInsertOrUpdateQueries(tableName, {"bookmark" : "''", "value" : "'Directory , File Name  ;right;102'", "type" : "'subfolder'"}, ["value"])
        sqlQueries += getAmendedSQLInsertOrUpdateQueries(tableName, {"bookmark" : "''", "value" : "'File Name , Directory  ;right;102'", "type" : "'subfolder'"}, ["value"])
        sqlQueries += getAmendedSQLInsertOrUpdateQueries(tableName, {"bookmark" : "''", "value" : "'Directory Name , Directory  ;right;102'", "type" : "'cover'"}, ["value"])
        sqlQueries += getAmendedSQLInsertOrUpdateQueries(tableName, {"bookmark" : "''", "value" : "'Source Cover , Current Cover  ;right;102'", "type" : "'cover'"}, ["value"])
        sqlQueries += getAmendedSQLInsertOrUpdateQueries(tableName, {"bookmark" : "''", "value" : "'Destination Cover , Source Cover  ;right;102'", "type" : "'cover'"}, ["value"])
        sqlQueries += getAmendedSQLInsertOrUpdateQueries(tableName, {"bookmark" : "''", "value" : "'Destination Cover , Current Cover  ;right;102'", "type" : "'cover'"}, ["value"])
        return sqlQueries
        
        



## This file is part of HamsiManager.
## 
## Copyright (c) 2010 Murat Demir <mopened@gmail.com>      
##
## Hamsi Manager is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## Hamsi Manager is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with HamsiManager; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import Variables
from MyObjects import *
import Universals
import Dialogs
import InputOutputs
import Options
import Organizer

MyDialog, MyDialogType, MyParent = getMyDialog()

class Hasher(MyDialog):
    def __init__(self, _file=None):
        MyDialog.__init__(self, MyParent)
        if MyDialogType=="MDialog":
            if Universals.isActivePyKDE4==True:
                self.setButtons(MyDialog.NoDefault)
        elif MyDialogType=="MMainWindow":
            self.setObjectName("Hasher")
            Universals.MainWindow = self
        newOrChangedKeys = Universals.newSettingsKeys + Universals.changedDefaultValuesKeys
        wOptionsPanel = Options.Options(None, "hash", None, newOrChangedKeys)
        lblPathOfPackage = MLabel(translate("Hasher", "Path Of The File : "))
        lblHash = MLabel(translate("Hasher", "Hash Type : "))
        lblHashOutput = MLabel(translate("Hasher", "Hash Output : "))
        lblHashDigestFile = MLabel(translate("Hasher", "Hash Digest File : "))
        lblHashDigest = MLabel(translate("Hasher", "Hash Digest : "))
        self.teHashDigest = MTextEdit("")
        self.cbHash = MComboBox()
        self.cbHash.addItems(Variables.getHashTypes())
        self.cbHashOutput = MComboBox()
        self.cbHashOutput.addItems([translate("Hasher", "Only Show"), translate("Hasher", "File"), translate("Hasher", "Clipboard")])
        self.leHashDigestFile = MLineEdit(trForM(_file))
        self.pbtnHash = MPushButton(translate("Hasher", "Hash"))
        self.pbtnClose = MPushButton(translate("Hasher", "Close"))
        self.lePathOfPackage = MLineEdit(trForM(_file))
        self.pbtnHash.setToolTip(translate("Hasher", "Hash the selected file"))
        self.pbtnSelectProjectPath = MPushButton(translate("Hasher", "Browse"))
        self.pbtnSelectPackagePath = MPushButton(translate("Hasher", "Browse"))
        self.connect(self.pbtnSelectPackagePath,SIGNAL("clicked()"),self.selectPackagePath)
        self.connect(self.pbtnHash,SIGNAL("clicked()"),self.hash)
        self.connect(self.pbtnClose,SIGNAL("clicked()"),self.close)
        self.connect(self.cbHash,SIGNAL("currentIndexChanged(int)"),self.pathOfPackageChanged)
        self.connect(self.cbHashOutput,SIGNAL("currentIndexChanged(int)"),self.hashOutputChanged)
        self.connect(self.lePathOfPackage,SIGNAL("textChanged(const QString&)"),self.pathOfPackageChanged)
        self.teHashDigest.setMaximumHeight(80)
        pnlMain = MWidget(self)
        tabwTabs = MTabWidget()
        vblMain = MVBoxLayout(pnlMain)
        pnlMain2 = MWidget(tabwTabs)
        vblMain2 = MVBoxLayout(pnlMain2)
        HBox1 = MHBoxLayout()
        HBox1.addWidget(self.pbtnHash)
        HBox1.addWidget(self.pbtnClose)
        HBox2 = MHBoxLayout()
        HBox2.addWidget(self.lePathOfPackage)
        HBox2.addWidget(self.pbtnSelectPackagePath)
        HBox3 = MHBoxLayout()
        HBox3.addWidget(lblHashDigest)
        HBox3.addWidget(self.teHashDigest)
        HBox4 = MHBoxLayout()
        HBox4.addWidget(lblHash)
        HBox4.addWidget(self.cbHash)
        HBox5 = MHBoxLayout()
        HBox5.addWidget(lblHashOutput)
        HBox5.addWidget(self.cbHashOutput)
        HBox6 = MHBoxLayout()
        HBox6.addWidget(lblHashDigestFile)
        HBox6.addWidget(self.leHashDigestFile)
        vblMain2.addWidget(lblPathOfPackage)
        vblMain2.addLayout(HBox2)
        vblMain2.addLayout(HBox4)
        vblMain2.addLayout(HBox3)
        vblMain2.addLayout(HBox5)
        vblMain2.addLayout(HBox6)
        vblMain2.addStretch(1)
        vblMain2.addLayout(HBox1)
        tabwTabs.addTab(pnlMain2, translate("Hasher", "Hash"))
        tabwTabs.addTab(wOptionsPanel, translate("Hasher", "Quick Options"))
        vblMain.addWidget(tabwTabs)
        self.pathOfPackageChanged("")
        self.hashOutputChanged(self.cbHashOutput.currentIndex())
        if MyDialogType=="MDialog":
            if Universals.isActivePyKDE4==True:
                self.setMainWidget(pnlMain)
            else:
                self.setLayout(vblMain)
        elif MyDialogType=="MMainWindow":
            self.setCentralWidget(pnlMain)
            moveToCenter(self)
        self.setWindowTitle(translate("Hasher", "Hasher"))
        self.setWindowIcon(MIcon("Images:hash.png"))
        self.show()
                        
    def closeEvent(self, _event):
        MApplication.setQuitOnLastWindowClosed(True)
    
    def pathOfPackageChanged(self, _value):
        try:
            self.teHashDigest.setText("")
            packageExtension =  "." + str(self.cbHash.currentText()).lower()
            self.leHashDigestFile.setText(self.lePathOfPackage.text() + packageExtension)  
        except:
            import ReportBug
            error = ReportBug.ReportBug()
            error.show()  
    
    def hashOutputChanged(self, _value):
        if _value==1:
            self.leHashDigestFile.setEnabled(True)
        else:
            self.leHashDigestFile.setEnabled(False)
    
    def hash(self):
        sourceFile = str(self.lePathOfPackage.text())
        if InputOutputs.IA.checkSource(sourceFile, "file"):
            hashType = str(self.cbHash.currentText())
            if hashType!=None:
                hashDigestContent = InputOutputs.IA.getHashDigest(sourceFile, hashType)
                if hashDigestContent!=False:
                    self.teHashDigest.setText(trForM(hashDigestContent))
                    if self.cbHashOutput.currentIndex()==1:
                        if InputOutputs.IA.createHashDigestFile(sourceFile, str(self.leHashDigestFile.text()), hashType, False, hashDigestContent):
                            Dialogs.show(translate("Hasher", "Hash Digest File Created"),
                                        str(translate("Hasher", "Hash digest writed into %s")) % str(self.leHashDigestFile.text()))
                        else:
                            Dialogs.showError(translate("Hasher", "Hash Digest File Is Not Created"),
                                        translate("Hasher", "Hash digest file not cteated."))
                    elif self.cbHashOutput.currentIndex()==2:
                            MApplication.clipboard().setText(trForM(hashDigestContent))
                            Dialogs.show(translate("Hasher", "Hash Digest Copied To Clipboard"),
                                        str(translate("Hasher", "Hash digest copied to clipboard.Hash digest is : <br>%s")) % hashDigestContent)
                else:
                    Dialogs.showError(translate("Hasher", "Hash Digest Is Not Created"),
                                    translate("Hasher", "Hash digest not cteated."))
        
    def selectPackagePath(self):
        try:
            self.teHashDigest.setText("")
            PathOfPackage = MFileDialog.getOpenFileName(self,
                        translate("Hasher", "Please Select The Pack To Be Created"), self.lePathOfPackage.text(),
                        translate("Hasher", "All Files (*.*)"))
            if PathOfPackage!="":
                self.lePathOfPackage.setText(PathOfPackage)    
        except:
            import ReportBug
            error = ReportBug.ReportBug()
            error.show() 
    
    
                



## This file is part of HamsiManager.
## 
## Copyright (c) 2010 Murat Demir <mopened@gmail.com>      
##
## Hamsi Manager is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## Hamsi Manager is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with HamsiManager; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import Variables
from MyObjects import *
import Universals
import Dialogs
import InputOutputs
import Options
import Organizer

MyDialog, MyDialogType, MyParent = getMyDialog()

class Packager(MyDialog):
    def __init__(self, _directory):
        MyDialog.__init__(self, MyParent)
        if MyDialogType=="MDialog":
            if Universals.isActivePyKDE4==True:
                self.setButtons(MyDialog.NoDefault)
        elif MyDialogType=="MMainWindow":
            self.setObjectName("Packager")
            Universals.MainWindow = self
        newOrChangedKeys = Universals.newSettingsKeys + Universals.changedDefaultValuesKeys
        wOptionsPanel = Options.Options(None, "pack", None, newOrChangedKeys)
        lblPleaseSelect = MLabel(translate("Packager", "Path Of The Directory"))
        lblPathOfPackage = MLabel(translate("Packager", "Path Of The Pack"))
        lblPackageType = MLabel(translate("Packager", "Package Compression Type : "))
        lblHash = MLabel(translate("Packager", "Hash : "))
        lblHashOutput = MLabel(translate("Packager", "Hash Output : "))
        lblHashDigestFile = MLabel(translate("Packager", "Hash Digest File : "))
        self.cbPackageType = MComboBox()
        self.cbPackageType.addItems([translate("Packager", "Archive Without Compression"),
                                    ".tar.gz", ".tar.bz2", ".amarokscript.tar.gz"])
        self.cbPackageType.setCurrentIndex(1)
        self.cbHash = MComboBox()
        self.cbHash.addItems([translate("Packager", "No Hash")] + Variables.getHashTypes())
        self.cbHashOutput = MComboBox()
        self.cbHashOutput.addItems([translate("Packager", "File"), translate("Packager", "Clipboard")])
        self.leHashDigestFile = MLineEdit(trForM(_directory))
        self.pbtnClearAndPack = MPushButton(translate("Packager", "Clear And Pack"))
        self.pbtnClear = MPushButton(translate("Packager", "Clear"))
        self.pbtnPack = MPushButton(translate("Packager", "Pack"))
        self.pbtnClose = MPushButton(translate("Packager", "Close"))
        self.lePathOfProject = MLineEdit(trForM(_directory))
        self.lePathOfPackage = MLineEdit(trForM(_directory))
        self.pbtnClearAndPack.setToolTip(translate("Packager", "Do not will cleared directory you selected but unnecessary files and directories package will not."))
        self.pbtnClear.setToolTip(translate("Packager", "Directory you selected will is cleared"))
        self.pbtnPack.setToolTip(translate("Packager", "Directory you selected will is packed. (Do not will Cleared)"))
        self.packageTypeChanged()
        self.pbtnSelectProjectPath = MPushButton(translate("Packager", "Browse"))
        self.pbtnSelectPackagePath = MPushButton(translate("Packager", "Browse"))
        self.connect(self.cbPackageType, SIGNAL("currentIndexChanged(int)"), self.packageTypeChanged)
        self.connect(self.pbtnSelectProjectPath,SIGNAL("clicked()"),self.selectProjectPath)
        self.connect(self.pbtnSelectPackagePath,SIGNAL("clicked()"),self.selectPackagePath)
        self.connect(self.pbtnClearAndPack,SIGNAL("clicked()"),self.ClearAndPack)
        self.connect(self.pbtnClear,SIGNAL("clicked()"),self.Clear)
        self.connect(self.pbtnPack,SIGNAL("clicked()"),self.Pack)
        self.connect(self.pbtnClose,SIGNAL("clicked()"),self.close)
        self.connect(self.cbHash,SIGNAL("currentIndexChanged(int)"),self.hashChanged)
        self.connect(self.cbHashOutput,SIGNAL("currentIndexChanged(int)"),self.hashOutputChanged)
        self.connect(self.lePathOfProject,SIGNAL("textChanged(const QString&)"),self.pathOfProjectChanged)
        self.connect(self.lePathOfPackage,SIGNAL("textChanged(const QString&)"),self.pathOfPackageChanged)
        pnlMain = MWidget(self)
        tabwTabs = MTabWidget()
        vblMain = MVBoxLayout(pnlMain)
        pnlMain2 = MWidget(tabwTabs)
        vblMain2 = MVBoxLayout(pnlMain2)
        HBox = MHBoxLayout()
        HBox.addWidget(self.lePathOfProject)
        HBox.addWidget(self.pbtnSelectProjectPath)
        HBox1 = MHBoxLayout()
        HBox1.addWidget(self.pbtnClear)
        HBox1.addWidget(self.pbtnPack)
        HBox1.addWidget(self.pbtnClearAndPack)
        HBox1.addWidget(self.pbtnClose)
        HBox2 = MHBoxLayout()
        HBox2.addWidget(self.lePathOfPackage)
        HBox2.addWidget(self.pbtnSelectPackagePath)
        HBox3 = MHBoxLayout()
        HBox3.addWidget(lblPackageType)
        HBox3.addWidget(self.cbPackageType)
        HBox4 = MHBoxLayout()
        HBox4.addWidget(lblHash)
        HBox4.addWidget(self.cbHash)
        HBox5 = MHBoxLayout()
        HBox5.addWidget(lblHashOutput)
        HBox5.addWidget(self.cbHashOutput)
        HBox6 = MHBoxLayout()
        HBox6.addWidget(lblHashDigestFile)
        HBox6.addWidget(self.leHashDigestFile)
        vblMain2.addWidget(lblPleaseSelect)
        vblMain2.addLayout(HBox)
        vblMain2.addWidget(lblPathOfPackage)
        vblMain2.addLayout(HBox2)
        vblMain2.addLayout(HBox3)
        vblMain2.addLayout(HBox4)
        vblMain2.addLayout(HBox5)
        vblMain2.addLayout(HBox6)
        vblMain2.addStretch(1)
        vblMain2.addLayout(HBox1)
        tabwTabs.addTab(pnlMain2, translate("Packager", "Pack"))
        tabwTabs.addTab(wOptionsPanel, translate("Packager", "Quick Options"))
        vblMain.addWidget(tabwTabs)
        self.hashChanged(self.cbHash.currentIndex())
        if MyDialogType=="MDialog":
            if Universals.isActivePyKDE4==True:
                self.setMainWidget(pnlMain)
            else:
                self.setLayout(vblMain)
        elif MyDialogType=="MMainWindow":
            self.setCentralWidget(pnlMain)
            moveToCenter(self)
        self.setWindowTitle(translate("Packager", "Packager"))
        self.setWindowIcon(MIcon("Images:pack.png"))
        self.show()
                        
    def closeEvent(self, _event):
        MApplication.setQuitOnLastWindowClosed(True)
    
    def pathOfProjectChanged(self, _value):
        try:
            if self.cbPackageType.currentIndex()!=0:
                packageExtension = self.cbPackageType.currentText()
            else:
                packageExtension = ".tar"   
            self.lePathOfPackage.setText(self.lePathOfProject.text()+packageExtension)  
        except:
            import ReportBug
            error = ReportBug.ReportBug()
            error.show()  
    
    def pathOfPackageChanged(self, _value):
        try:
            if self.cbHash.currentIndex()==0:
                packageExtension = "~"
            else:
                packageExtension =  "." + str(self.cbHash.currentText()).lower()
            self.leHashDigestFile.setText(self.lePathOfPackage.text()+packageExtension)  
        except:
            import ReportBug
            error = ReportBug.ReportBug()
            error.show()  
    
    def hashChanged(self, _value):
        if _value==0:
            self.cbHashOutput.setEnabled(False)
            self.leHashDigestFile.setEnabled(False)
        else:
            self.cbHashOutput.setEnabled(True)
            self.hashOutputChanged(self.cbHashOutput.currentIndex())
        self.pathOfPackageChanged("")
    
    def hashOutputChanged(self, _value):
        if _value==0:
            self.leHashDigestFile.setEnabled(True)
        else:
            self.leHashDigestFile.setEnabled(False)
    
    def createHashDigest(self):
        if self.cbHash.currentIndex()==0:
            hashType = None
        else:
            hashType =  str(self.cbHash.currentText())
        if hashType!=None:
            if self.cbHashOutput.currentIndex()==0:
                if InputOutputs.IA.createHashDigestFile(str(self.lePathOfPackage.text()), str(self.leHashDigestFile.text()), hashType, False):
                    Dialogs.show(translate("Packager", "Hash Digest File Created"),
                                str(translate("Packager", "Hash digest writed into %s")) % str(self.leHashDigestFile.text()))
                else:
                    Dialogs.showError(translate("Packager", "Hash Digest File Is Not Created"),
                                translate("Packager", "Hash digest file not cteated."))
            else:
                hashDigestContent = InputOutputs.IA.getHashDigest(str(self.lePathOfPackage.text()), hashType)
                if hashDigestContent!=False:
                    MApplication.clipboard().setText(trForM(hashDigestContent))
                    Dialogs.show(translate("Packager", "Hash Digest Copied To Clipboard"),
                                str(translate("Packager", "Hash digest copied to clipboard.Hash digest is : <br>%s")) % hashDigestContent)
                else:
                    Dialogs.showError(translate("Packager", "Hash Digest Is Not Created"),
                                translate("Packager", "Hash digest not cteated."))
                    
    
    def ClearAndPack(self):
        try:
            Universals.isCanBeShowOnMainWindow = False
            import tempfile, random
            tempDir = tempfile.gettempdir() + "/HamsiManager-" + str(random.randrange(0, 1000000))
            PathOfProject = str(self.lePathOfProject.text())
            InputOutputs.IA.copyFileOrDir(PathOfProject, tempDir+"/"+InputOutputs.IA.getBaseName(PathOfProject))
            InputOutputs.IA.clearPackagingDirectory(tempDir, True, True)
            if InputOutputs.IA.makePack(str(self.lePathOfPackage.text()), self.getPackageType(), tempDir):
                InputOutputs.IA.removeFileOrDir(tempDir, True)
                self.createHashDigest()
                Dialogs.show(translate("Packager", "Project Is Packed"),
                            translate("Packager", "You can now start sharing it."))
            if Universals.getBoolValue("isCloseOnCleanAndPackage"):
                self.close()
            Universals.isCanBeShowOnMainWindow = True
        except:
            import ReportBug
            error = ReportBug.ReportBug()
            error.show()   
    
    def Clear(self):
        try:
            Universals.isCanBeShowOnMainWindow = False
            answer = Dialogs.ask(translate("Packager", "Your Files Will Be Removed"),
                    str(translate("Packager", "The files in the \"%s\" folder will be cleared according to the criteria you set.<br>"+
                    "This action will delete the files completely, without any chance to recover.<br>"+
                    "Are you sure you want to perform the action?")) % Organizer.getLink(str(self.lePathOfProject.text())))
            if answer==Dialogs.Yes:
                if InputOutputs.IA.clearPackagingDirectory(str(self.lePathOfProject.text()), True, True):
                    Dialogs.show(translate("Packager", "Project Is Cleared"),
                                translate("Packager", "You can now pack your project."))
            Universals.isCanBeShowOnMainWindow = True
        except:
            import ReportBug
            error = ReportBug.ReportBug()
            error.show() 
    
    def Pack(self):
        try:
            Universals.isCanBeShowOnMainWindow = False
            if InputOutputs.IA.makePack(str(self.lePathOfPackage.text()), 
                                self.getPackageType(), str(self.lePathOfProject.text())):
                self.createHashDigest()
                Dialogs.show(translate("Packager", "Project Is Packed"),
                            translate("Packager", "You can now share your project."))
            Universals.isCanBeShowOnMainWindow = True
        except:
            import ReportBug
            error = ReportBug.ReportBug()
            error.show()   
    
    def getPackageType(self):
        if self.cbPackageType.currentIndex()!=0:
            return str(self.cbPackageType.currentText()).split(".")[-1]
        else:
            return ""
    
    def selectProjectPath(self):
        try:
            self.cbPackageType.setEnabled(True)
            ProjectPath = MFileDialog.getExistingDirectory(self,
                            translate("Packager", "Please Select Project Folder"),self.lePathOfProject.text())
            if ProjectPath!="":
                self.lePathOfProject.setText(ProjectPath)  
            self.packageTypeChanged()
        except:
            import ReportBug
            error = ReportBug.ReportBug()
            error.show() 
        
    def packageTypeChanged(self):
        try:
            packageName = self.lePathOfPackage.text()
            if packageName[-4:]==".tar":
                packageName = packageName[:-4]
            elif packageName[-20:]==".amarokscript.tar.gz":
                packageName = packageName[:-20]
            elif packageName[-7:]==".tar.gz":
                packageName = packageName[:-7]
            elif packageName[-8:]==".tar.bz2":
                packageName = packageName[:-8]
            if self.cbPackageType.currentIndex()!=0:
                packageExtension = self.cbPackageType.currentText()
            else:
                packageExtension = ".tar"   
            self.lePathOfPackage.setText(packageName+packageExtension)  
        except:
            import ReportBug
            error = ReportBug.ReportBug()
            error.show() 
        
    def selectPackagePath(self):
        try:
            if self.cbPackageType.currentIndex()!=0:
                packageExtension = self.cbPackageType.currentText()
            else:
                packageExtension = ".tar"   
            PathOfPackage = MFileDialog.getSaveFileName(self,
                        translate("Packager", "Please Select The Pack To Be Created"),self.lePathOfPackage.text(),
                        str(translate("Packager", "Archive Files (*%s)")) % (packageExtension))
            if PathOfPackage!="":
                self.lePathOfPackage.setText(PathOfPackage)    
        except:
            import ReportBug
            error = ReportBug.ReportBug()
            error.show() 
    
    
                



## This file is part of HamsiManager.
## 
## Copyright (c) 2010 Murat Demir <mopened@gmail.com>      
##
## Hamsi Manager is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## Hamsi Manager is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with HamsiManager; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import Variables
from MyObjects import *
import Dialogs
import ReportBug

class SearchEngines(MMenu):
    def __init__(self,  _parent, _isCheckSingleFile=False):
        self.isCheckSingleFile = _isCheckSingleFile
        MMenu.__init__(self, _parent)
        self.setTitle(translate("SearchEngines", "Verify On The Internet"))
        self.actions, self.searchDepths = [], []
        self.searchEnginesNames = Variables.getSearchEnginesNames()
        isAnyAvailable = False
        for sEngine in self.searchEnginesNames:
            sEngineModule = __import__("SearchEngines." + sEngine, globals(), locals(), ["isAvailable", "pluginName"], -1)
            if sEngineModule.isAvailable:
                isAnyAvailable = True
                self.actions.append(MAction(trForUI(sEngineModule.pluginName), self))
                self.actions[-1].setObjectName(str(len(self.actions)-1))
                self.addAction(self.actions[-1])
                if sEngineModule.pluginName=="MusicBrainz":
                    self.mSearchDepth = MMenu()
                    self.mSearchDepth.setTitle(translate("SearchEngines", "MusicBrainz (Choose Search Depth)"))
                    for no in range(1, 12):
                        self.searchDepths.append(MAction(str(no), self.mSearchDepth))
                        self.searchDepths[-1].setObjectName(str(len(self.actions)-1)+"-MusicBrainz-"+str(no))
                        self.mSearchDepth.addAction(self.searchDepths[-1])
                    self.addMenu(self.mSearchDepth )
        if isAnyAvailable==False:
            self.actions.append(MAction(translate("SearchEngines", "You Have Not Any Search Engine"), self))
            self.actions[-1].setObjectName(translate("SearchEngines", "You Have Not Any Search Engine"))
            self.addAction(self.actions[-1])
        MObject.connect(self,SIGNAL("triggered(QAction *)"),self.triggered)

    def triggered(self, _action):
        try:
            if _action.objectName()==translate("SearchEngines", "You Have Not Any Search Engine"):
                Dialogs.show(translate("SearchEngines", "You Have Not Any Search Engine"), 
                                translate("SearchEngines", "Not found any search engine in your system. Please install a search engine module. Now supporting only musicbrainz module (python-musicbrainz2)."))
            else:
                if self.parent().rowCount()!=0:
                    selectedSearchDepth = 3
                    if str(_action.objectName()).find("-MusicBrainz-")!=-1:
                        info = _action.objectName().split("-")
                        engine = self.searchEnginesNames[int(info[0])]
                        selectedSearchDepth = info[2]
                    else:
                        engine = self.searchEnginesNames[int(_action.objectName())]
                    sEngineModule = __import__("SearchEngines." + engine, globals(), locals(), ["Search"], -1)
                    sEngineModule.Search(self.parent(), self.isCheckSingleFile, selectedSearchDepth)
                else:
                    Dialogs.show(translate("SearchEngines", "Table Is Empty"), 
                                translate("SearchEngines", "Nothing to be done because the table is empty."))
        except:
            error = ReportBug.ReportBug()
            error.show()
        
        



## This file is part of HamsiManager.
## 
## Copyright (c) 2010 Murat Demir <mopened@gmail.com>      
##
## Hamsi Manager is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## Hamsi Manager is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with HamsiManager; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import unicodedata
import string
import Variables
import Settings
import Universals
if Variables.isPython3k:
    from urllib.parse import unquote, quote
else:
    from urllib import unquote, quote

class Organizer:
    """Music tags, filenames, Turkish characters etc. will be arranged through this class
    """
    global applySpecialCommand, emend, whatDoesSpecialCommandDo,searchAndReplaceTable, fillTable, clearTable, makeCorrectCaseSensitive, correctCaseSensitiveTable, searchAndReplace, clear, correctCaseSensitive, searchAndReplaceFromSearchAndReplaceTable, getLink, getIconName
    
    def emend(_inputString, _type="text", _isCorrectCaseSensitive=True, _isRichText=False):
        _inputString = str(_inputString)
        if len(_inputString)==0: return ""
        if Universals.getBoolValue("isClearFirstAndLastSpaceChars"):
            _inputString = _inputString.strip()
        if Universals.getBoolValue("isEmendIncorrectChars"):
            try:_inputString = Universals.trUnicode(_inputString)
            except:_inputString = Universals.trUnicode(_inputString, "iso-8859-9")
            replacementChars = Universals.getUtf8Data("replacementChars")
            _inputString = unquote(_inputString)
            for oldChar, newChar in replacementChars.items():
                _inputString = _inputString.replace(oldChar,newChar)
        _inputString = str(Universals.trDecode(_inputString, "utf-8", "ignore"))
        if len(_inputString)==0: return ""
        preString, extString, ext2String = "", "", ""
        if _type=="file" or _type=="directory":
            if _inputString[-1]=="/":
                _inputString = _inputString[:-1]
                ext2String = "/"
            if _inputString.find("/")!=-1:
                tStr = _inputString.rsplit("/", 1)
                preString = tStr[0] + "/"
                _inputString = tStr[1]
            if _type=="file":
                if _inputString.find(".")!=-1:
                    if Universals.MySettings["fileExtesionIs"]==Variables.fileExtesionIsKeys[0]:
                        tStr = _inputString.split(".", 1)
                        extString = "." + tStr[1]
                        _inputString = tStr[0]
                    elif Universals.MySettings["fileExtesionIs"]==Variables.fileExtesionIsKeys[1]:
                        tStr = _inputString.rsplit(".", 1)
                        extString = "." + tStr[1]
                        _inputString = tStr[0]
            if _isCorrectCaseSensitive:
                extString = makeCorrectCaseSensitive(extString, Universals.MySettings["validSentenceStructureForFileExtension"])
                _inputString = makeCorrectCaseSensitive(_inputString, Universals.MySettings["validSentenceStructureForFile"])
            if Universals.MySettings["fileReNamerType"]==Variables.fileReNamerTypeNamesKeys[1] or Universals.MySettings["fileReNamerType"]==Variables.fileReNamerTypeNamesKeys[2]:
                _inputString = ''.join(c for c in unicodedata.normalize('NFKD', Universals.trUnicode(_inputString)) if unicodedata.category(c) != 'Mn')
                _inputString = str(Universals.trEncode(_inputString, "utf-8", "ignore")).replace(Universals.getUtf8Data("little+I"), "i")
            oldChars, newChars = [], []
            if Universals.MySettings["fileReNamerType"]==Variables.fileReNamerTypeNamesKeys[1]:
                oldChars = [" "]
                newChars = ["_"]
            for x, oldChar in enumerate(oldChars):
                _inputString = _inputString.replace(oldChar,newChars[x])
            if Universals.MySettings["fileReNamerType"]==Variables.fileReNamerTypeNamesKeys[1]:
                _inputString = quote(_inputString)
            if Universals.getBoolValue("isCorrectFileNameWithSearchAndReplaceTable"):
                _inputString = searchAndReplaceFromSearchAndReplaceTable(_inputString)
        else:
            if _isCorrectCaseSensitive:
                _inputString = makeCorrectCaseSensitive(_inputString, Universals.MySettings["validSentenceStructure"])
            _inputString = searchAndReplaceFromSearchAndReplaceTable(_inputString)
        if _isRichText==False:
            if Universals.getBoolValue("isCorrectDoubleSpaceChars"):
                isFinded=_inputString.find("  ")
                while isFinded!=-1:
                    _inputString=_inputString.replace("  "," ")
                    isFinded=_inputString.find("  ")
        return preString + _inputString + extString + ext2String
    
    def makeCorrectCaseSensitive(_inputString, _cbCharacterType):
        if _cbCharacterType==Variables.validSentenceStructureKeys[0]:
            return string.capwords(Universals.trUnicode(_inputString))
        elif _cbCharacterType==Variables.validSentenceStructureKeys[1]:
            return string.lower(Universals.trUnicode(_inputString))
        elif _cbCharacterType==Variables.validSentenceStructureKeys[2]:
            return string.upper(Universals.trUnicode(_inputString))
        elif _cbCharacterType==Variables.validSentenceStructureKeys[3]:
            return string.capitalize(Universals.trUnicode(_inputString))
        else :
            return str(Universals.trUnicode(_inputString))
    
    def getLink(_stringPath):
        _stringPath = str(_stringPath)
        return "<a href=\"file://%s\" target=\"_blank\">%s</a>" % (_stringPath, _stringPath)
    
    def getIconName(_artist, _album, _year, _genre):
        iconName = Universals.MySettings["iconNameFormat"]
        iconName = iconName.replace(Variables.iconNameFormatKeys[0], _artist)
        iconName = iconName.replace(Variables.iconNameFormatKeys[1], _album)
        iconName = iconName.replace(Variables.iconNameFormatKeys[2], _year)
        iconName = iconName.replace(Variables.iconNameFormatKeys[3], _genre)
        return iconName + "." + Universals.MySettings["iconFileType"]

    def searchAndReplaceFromSearchAndReplaceTable(_oldString):
        import Databases
        newString = _oldString
        for info in Databases.SearchAndReplaceTable.fetchAll():
            if info[3]==1:
                isCaseSensitive, isRegExp = False, False
                if info[4]==1:
                    isCaseSensitive = True
                if info[5]==1:
                    isRegExp = True
                newString = searchAndReplace(newString, [info[1]], [info[2]], isCaseSensitive, isRegExp)
        return newString
    
    def applySpecialCommand(_splitPointer, _whereIsSplitPointer, _command, _SpecialTools):
        import Tables
        blok = _command.split(",")
        changings = blok[0].split(_splitPointer)
        changers = blok[1].split(_splitPointer)
        changingColumns,changerColumns=[],[]
        Tables.isChangeHiddenColumn,Tables.isAskShowHiddenColumn=True,True
        for columnName in changings:
            columnName=columnName.strip()
            for no,column in enumerate(Universals.MainWindow.Table.tableColumnsKey):
                if columnName==column:
                    if Tables.checkHiddenColumn(no)==False:
                        return False
                    changingColumns.append(no)
        if len(changingColumns)==0:
            return False
        for columnName in changers:
            columnName=columnName.strip()
            for no,column in enumerate(Universals.MainWindow.Table.tableColumnsKey):
                if columnName==column:
                    if Tables.checkHiddenColumn(no)==False:
                        return False
                    changerColumns.append(no)
        if len(changerColumns)==0:
            return False
        if Tables.isChangeHiddenColumn==True:
            if _whereIsSplitPointer=="right":
                for rowNo in range(Universals.MainWindow.Table.rowCount()):
                    if Universals.MainWindow.Table.isChangableItem(rowNo, changingColumns[0]):
                        newString=""
                        for changerColumnNo in changerColumns:
                            if str(Universals.MainWindow.Table.item(rowNo,changerColumnNo).text()) != "-----":
                                newString+=" "+_splitPointer+" "+str(Universals.MainWindow.Table.item(rowNo,changerColumnNo).text())
                        newString = emend(newString[2:])
                        if newString!="":
                            for uzanti in Universals.getListFromStrint(Universals.MySettings["musicExtensions"]):
                                if newString.split(".")[-1].lower() == str(uzanti) :
                                    newString = newString[:-len(newString.split(".")[-1])-1]
                            if _SpecialTools.btChange.isChecked()==True:
                                pass
                            elif _SpecialTools.tbAddToBefore.isChecked()==True:
                                newString += str(Universals.MainWindow.Table.item(rowNo,changingColumns[0]).text())
                            elif _SpecialTools.tbAddToAfter.isChecked()==True:
                                newString = str(Universals.MainWindow.Table.item(rowNo,changingColumns[0]).text()) + newString
                            Universals.MainWindow.Table.item(rowNo,changingColumns[0]).setText(trForUI(newString.strip()))
            else:
                for rowNo in range(Universals.MainWindow.Table.rowCount()):
                    newString = str(Universals.MainWindow.Table.item(rowNo,changerColumns[0]).text())
                    if newString!="-----":
                        for uzanti in Universals.getListFromStrint(Universals.MySettings["musicExtensions"]):
                            if newString.split(".")[-1].lower() == str(uzanti) :
                                newString = newString[:-len(newString.split(".")[-1])-1]
                        newStrings = ["","","","","","","",""]
                        newString = newString.split(_splitPointer)
                        for stringNo in range(0,len(newString)):
                            newStrings[stringNo] = newString[stringNo]
                        stringNo=0
                        for changingColumnNo in changingColumns:
                            if Universals.MainWindow.Table.isChangableItem(rowNo, changingColumnNo):
                                if _SpecialTools.btChange.isChecked()==True:
                                    pass
                                elif _SpecialTools.tbAddToBefore.isChecked()==True:
                                    newStrings[stringNo] += str(Universals.MainWindow.Table.item(rowNo,changingColumnNo).text())
                                elif _SpecialTools.tbAddToAfter.isChecked()==True:
                                    newStrings[stringNo] = str(Universals.MainWindow.Table.item(rowNo,changingColumnNo).text()) + newStrings[stringNo]
                                Universals.MainWindow.Table.item(rowNo,changingColumnNo).setText(trForUI(newStrings[stringNo].strip()))
                            stringNo+=1
        
    def whatDoesSpecialCommandDo(_splitPointer, _whereIsSplitPointer, _command, _isCorrect=False, _isReturnDetails=False):
        import Dialogs
        from MyObjects import translate
        if _command[-2:]!="- " and _command[-2:]!=", " and _command.find(",")!=-1:
            _command = _command.split(",")
            changings = _command[0].split(_splitPointer)
            changers = _command[1].split(_splitPointer)
            if _whereIsSplitPointer=="right":
                details=""
                for changerColumnName in changers:
                    if len(changers)>1:
                        if changerColumnName==changers[-2]:
                            appended= str(translate("Organizer", " and "))
                        elif changerColumnName==changers[-1]:
                            appended=""
                        else:
                            appended=","
                    else:
                        appended = ""
                    details+=str(Universals.MainWindow.Table.getColumnNameFromKey(changerColumnName.strip()))+appended
                details+=str(translate("Organizer", " \"%s\" is concatenated with and is set as \"%s\".")) % (_splitPointer, str(Universals.MainWindow.Table.getColumnNameFromKey(changings[0].strip()))) 
                if len(changers)==1:
                    details = str(translate("Organizer", "\"%s\" \"%s\" are set.")) % (str(Universals.MainWindow.Table.getColumnNameFromKey(changers[0].strip())), str(Universals.MainWindow.Table.getColumnNameFromKey(changings[0].strip())))
                if _isCorrect==True:
                    return True
                if _isReturnDetails==True:
                    return details
                Dialogs.show(translate("Organizer", "What Does This Command Do?"),trForUI(details))
                
            else:
                details = ""
                for changingColumnName in changings:
                    if len(changings)>1:
                        if changingColumnName==changings[-2]:
                            appended=str(translate("Organizer", " and "))
                        elif changingColumnName==changings[-1]:
                            appended=""
                        else:
                            appended=","
                    else:
                        appended = ""
                    details+= str(Universals.MainWindow.Table.getColumnNameFromKey(changingColumnName.strip()))+appended
                details = str(translate("Organizer", "\"%s\" \"%s\" hyphenates from the apostrophe and sets each piece as \"%s\".")) % (str(Universals.MainWindow.Table.getColumnNameFromKey(changers[0].strip())), _splitPointer, details)
                if _isCorrect==True:
                    return True
                if _isReturnDetails==True:
                    return details
                Dialogs.show(translate("Organizer", "What Does This Command Do?"),details)
        else:
            if _command.find(",")==-1:
                if _isReturnDetails==True:
                    return (translate("Organizer", "You have to add a \",\"(comma) to your command!.."))
                Dialogs.showError(translate("Organizer", "Missing Command"),
                             translate("Organizer", "You have to add a \",\"(comma) to your command!.."))
                return False
            else:
                if _isReturnDetails==True:
                    return (translate("Organizer", "You have to add one (more) \"Column\"!.."))
                Dialogs.showError(translate("Organizer", "Missing Command"),
                             translate("Organizer", "You have to add one (more) \"Column\"!.."))
                return False
    
    def searchAndReplaceTable(_searchStrings,_replaceStrings, _SpecialTools):
        from MyObjects import trForUI
        searchStrings=_searchStrings.split(";")
        replaceStrings=_replaceStrings.split(";")
        for filterNo in range(0,len(searchStrings)):
            if _SpecialTools.btChange.isChecked()==True:
                pass
            elif _SpecialTools.tbAddToBefore.isChecked()==True:
                replaceStrings[filterNo] += searchStrings[filterNo]
            elif _SpecialTools.tbAddToAfter.isChecked()==True:
                replaceStrings[filterNo] = searchStrings[filterNo] + replaceStrings[filterNo]
        while len(replaceStrings)!=len(searchStrings):
            replaceStrings.append("")
        if _SpecialTools.searchAndReplace.columns.currentIndex()==0:
            columns = list(range(0,Universals.MainWindow.Table.columnCount()))
        else:
            columns = [_SpecialTools.searchAndReplace.columns.currentIndex()-1]
        for columnNo in columns:
            if Universals.MainWindow.Table.isColumnHidden(columnNo)==True:
                continue
            for rowNo in range(Universals.MainWindow.Table.rowCount()):
                if Universals.MainWindow.Table.isChangableItem(rowNo, columnNo, None, True):
                    newString = str(Universals.MainWindow.Table.item(rowNo,columnNo).text())
                    newString = trForUI(newString)
                    myString = ""
                    informationSectionX = _SpecialTools.cbInformationSectionX.value()
                    informationSectionY = _SpecialTools.cbInformationSectionY.value()
                    isCaseSensitive = _SpecialTools.searchAndReplace.cckbCaseSensitive.isChecked()
                    isRegExp = _SpecialTools.searchAndReplace.cckbRegExp.isChecked()
                    if _SpecialTools.cbInformationSection.currentIndex()==0:
                        myString = searchAndReplace(newString, searchStrings, 
                                               replaceStrings, isCaseSensitive, isRegExp)
                    elif _SpecialTools.cbInformationSection.currentIndex()==1:
                        myString = searchAndReplace(newString[:informationSectionX], searchStrings, 
                                               replaceStrings, isCaseSensitive, isRegExp)
                        myString += newString[informationSectionX:]
                    elif _SpecialTools.cbInformationSection.currentIndex()==2:
                        myString = newString[:informationSectionX]
                        myString += searchAndReplace(newString[informationSectionX:], searchStrings, 
                                                replaceStrings, isCaseSensitive, isRegExp)
                    elif _SpecialTools.cbInformationSection.currentIndex()==3:
                        myString = searchAndReplace(newString[:-informationSectionX], searchStrings, 
                                               replaceStrings, isCaseSensitive, isRegExp)
                        myString += newString[-informationSectionX:]
                    elif _SpecialTools.cbInformationSection.currentIndex()==4:
                        myString = newString[:-informationSectionX]
                        myString += searchAndReplace(newString[-informationSectionX:], searchStrings, 
                                                replaceStrings, isCaseSensitive, isRegExp)
                    elif _SpecialTools.cbInformationSection.currentIndex()==5:
                        myString = newString[:informationSectionX]
                        myString += searchAndReplace(newString[informationSectionX:informationSectionY], searchStrings, 
                                                replaceStrings, isCaseSensitive, isRegExp)
                        myString += newString[informationSectionY:]
                    elif _SpecialTools.cbInformationSection.currentIndex()==6:
                        myString = searchAndReplace(newString[:informationSectionX], searchStrings, 
                                                replaceStrings, isCaseSensitive, isRegExp)
                        myString += newString[informationSectionX:informationSectionY]
                        myString += searchAndReplace(newString[informationSectionY:], searchStrings, 
                                                replaceStrings, isCaseSensitive, isRegExp)
                    Universals.MainWindow.Table.item(rowNo,columnNo).setText(trForUI(myString))
    
    def searchAndReplace(_oldString, _searchStrings, _replaceStrings, _isCaseSensitive=True, _isRegExp=False):
        newString = _oldString
        for filterNo in range(0,len(_searchStrings)):
            if _searchStrings[filterNo]!="":
                if _isRegExp == True:
                    import re
                    if _isCaseSensitive ==True:
                        pattern = re.compile(Universals.trUnicode(_searchStrings[filterNo]), re.I | re.U)
                        newString = re.sub(pattern,Universals.trUnicode(_replaceStrings[filterNo]), Universals.trUnicode(newString))
                    else:
                        pattern = re.compile(Universals.trUnicode(_searchStrings[filterNo]))
                        newString = re.sub(pattern,Universals.trUnicode(_replaceStrings[filterNo]), Universals.trUnicode(newString))
                else:
                    if _isCaseSensitive ==True:
                        import re
                        pattern = re.compile(re.escape(Universals.trUnicode(_searchStrings[filterNo])), re.I | re.U)
                        newString = re.sub(pattern,Universals.trUnicode(_replaceStrings[filterNo]), Universals.trUnicode(newString))
                    else:
                        newString = newString.replace(_searchStrings[filterNo],_replaceStrings[filterNo])
        return newString
    
    def fillTable(_columnName, _SpecialTools,_newString=""):
        from MyObjects import trForUI
        import Tables
        Tables.isChangeHiddenColumn,Tables.isAskShowHiddenColumn=True,True
        for No, columnName in enumerate(Universals.MainWindow.Table.tableColumns):
            if str(_columnName) == str(columnName):
                columnNo=No
                break
        if Tables.checkHiddenColumn(columnNo,False)==False:
            return False
        if Tables.isChangeHiddenColumn==True:
            if _SpecialTools.fill.cbFillType.currentIndex()==1:
                _newString = int(_SpecialTools.fill.spStartDigit.value())-1
            for rowNo in range(Universals.MainWindow.Table.rowCount()):
                if Universals.MainWindow.Table.isChangableItem(rowNo, columnNo):
                    if _SpecialTools.fill.cbFillType.currentIndex()==1:
                        if _SpecialTools.fill.cbSort.currentIndex()==0:
                            _newString+=1
                        else:
                            _newString-=1
                        myString = str(_newString)
                        inNegative = False
                        if myString.find("-")!=-1:
                            myString = myString.replace("-", "")
                            inNegative = True
                        karakterSayisi = len(myString)
                        while karakterSayisi < int(_SpecialTools.fill.spCharNumberOfDigit.value()):
                            myString="0"+myString
                            karakterSayisi = len(myString)
                        if inNegative:
                            myString="-"+myString
                    else:
                        myString = str(_newString)
                    if _SpecialTools.btChange.isChecked()==True:
                        pass
                    elif _SpecialTools.tbAddToBefore.isChecked()==True:
                        myString += str(Universals.MainWindow.Table.item(rowNo,columnNo).text())
                    elif _SpecialTools.tbAddToAfter.isChecked()==True:
                        myString = str(Universals.MainWindow.Table.item(rowNo,columnNo).text()) + myString
                    Universals.MainWindow.Table.item(rowNo,columnNo).setText(trForUI(Universals.trUnicode(myString).title()))
                    
    def clearTable(_SpecialTools):
        from MyObjects import trForUI
        import Tables
        Tables.isChangeHiddenColumn,Tables.isAskShowHiddenColumn=True,True
        if _SpecialTools.clear.columns.currentIndex()==0:
            columns = list(range(0,Universals.MainWindow.Table.columnCount()))
        else:
            columns = [_SpecialTools.clear.columns.currentIndex()-1]
        for columnNo in columns:
            if Tables.checkHiddenColumn(columnNo,False)==False:
                continue
            for rowNo in range(Universals.MainWindow.Table.rowCount()):
                if Universals.MainWindow.Table.isChangableItem(rowNo, columnNo):
                    newString = str(Universals.MainWindow.Table.item(rowNo,columnNo).text())
                    newString = Universals.trDecode(newString, "utf-8")
                    informationSectionX = _SpecialTools.cbInformationSectionX.value()
                    informationSectionY = _SpecialTools.cbInformationSectionY.value()
                    isCaseSensitive = _SpecialTools.clear.cckbCaseSensitive.isChecked()
                    oldString = str(_SpecialTools.clear.leClear.text())
                    cbClearType = _SpecialTools.clear.cbClearType.currentText()
                    isRegExp = _SpecialTools.clear.cckbRegExp.isChecked()
                    if _SpecialTools.cbInformationSection.currentIndex()==0:
                        myString = clear(cbClearType, newString, 
                                               oldString, isCaseSensitive, isRegExp)
                    elif _SpecialTools.cbInformationSection.currentIndex()==1:
                        myString = clear(cbClearType, newString[:informationSectionX], 
                                               oldString, isCaseSensitive, isRegExp)
                        myString += newString[informationSectionX:]
                    elif _SpecialTools.cbInformationSection.currentIndex()==2:
                        myString = newString[:informationSectionX]
                        myString += clear(cbClearType, newString[informationSectionX:], 
                                                oldString, isCaseSensitive, isRegExp)
                    elif _SpecialTools.cbInformationSection.currentIndex()==3:
                        myString = clear(cbClearType, newString[:-informationSectionX], 
                                               oldString, isCaseSensitive, isRegExp)
                        myString += newString[-informationSectionX:]
                    elif _SpecialTools.cbInformationSection.currentIndex()==4:
                        myString = newString[:-informationSectionX]
                        myString += clear(cbClearType, newString[-informationSectionX:], 
                                                oldString, isCaseSensitive, isRegExp)
                    elif _SpecialTools.cbInformationSection.currentIndex()==5:
                        myString = newString[:informationSectionX]
                        myString += clear(cbClearType, newString[informationSectionX:informationSectionY], 
                                                oldString, isCaseSensitive, isRegExp)
                        myString += newString[informationSectionY:]
                    elif _SpecialTools.cbInformationSection.currentIndex()==6:
                        myString = clear(cbClearType, newString[:informationSectionX], 
                                                oldString, isCaseSensitive, isRegExp)
                        myString += newString[informationSectionX:informationSectionY]
                        myString += clear(cbClearType, newString[informationSectionY:], 
                                                oldString, isCaseSensitive, isRegExp)
                    Universals.MainWindow.Table.item(rowNo,columnNo).setText(trForUI(myString))
    
    def clear(_cbClearType, _oldString="", _searchString="", _isCaseSensitive=True, _isRegExp=False):
        from MyObjects import translate
        myString=""
        if _cbClearType==translate("SpecialTools", "All"):
            myString=""
        elif _cbClearType==translate("SpecialTools", "Letters"):
            for char in _oldString:
                if char.isalpha()==False:
                    myString+=char
        elif _cbClearType==translate("SpecialTools", "Numbers"):
            for char in _oldString:
                if char.isdigit()==False:
                    myString+=char
        elif _cbClearType==translate("SpecialTools", "Other Characters"):
            for char in _oldString:
                if char.isdigit()==True or char.isalpha()==True:
                    myString+=char
        elif _cbClearType==translate("SpecialTools", "Selected Text"):
            if _isRegExp == True:
                import re
                if _isCaseSensitive ==True:
                    pattern = re.compile(Universals.trUnicode(_searchString), re.I | re.U)
                    myString = re.sub(pattern,Universals.trUnicode(""), Universals.trUnicode(_oldString))
                else:
                    pattern = re.compile(Universals.trUnicode(_searchString))
                    myString = re.sub(pattern,Universals.trUnicode(""), Universals.trUnicode(_oldString))
            else:
                if _isCaseSensitive==True:
                    import re
                    pattern = re.compile(re.escape(Universals.trUnicode(_searchString)), re.I | re.U)
                    myString = re.sub(pattern,Universals.trUnicode(""), Universals.trUnicode(_oldString))
                else:
                    myString = _oldString.replace(_searchString,"")
        return myString
     
    def correctCaseSensitive(_inputString, _cbCharacterType, isCorrectText = False, _searchStrings=[], _isCaseSensitive=True, _isRegExp=False):
        newString = _inputString
        if isCorrectText:
            for filterNo in range(0,len(_searchStrings)):
                if _searchStrings[filterNo]!="":
                    if _isRegExp == True:
                        import re
                        if _isCaseSensitive ==True:
                            m = re.search(_searchStrings[filterNo], newString, re.I | re.U)
                            try:a = m.group(0)
                            except:return newString
                            pattern = re.compile(Universals.trUnicode(_searchStrings[filterNo]), re.I | re.U)
                            newString = re.sub(pattern,Universals.trUnicode(makeCorrectCaseSensitive(m.group(0), _cbCharacterType)), Universals.trUnicode(newString))
                        else:
                            m = re.search(_searchStrings[filterNo], newString)
                            try:a = m.group(0)
                            except:return newString
                            pattern = re.compile(Universals.trUnicode(_searchStrings[filterNo]))
                            newString = re.sub(pattern,Universals.trUnicode(makeCorrectCaseSensitive(m.group(0), _cbCharacterType)), Universals.trUnicode(newString))
                    else:
                        if _isCaseSensitive ==True:
                            import re
                            pattern = re.compile(re.escape(Universals.trUnicode(_searchStrings[filterNo])), re.I | re.U)
                            newString = re.sub(pattern,Universals.trUnicode(makeCorrectCaseSensitive(_searchStrings[filterNo], _cbCharacterType)), Universals.trUnicode(newString))
                        else:
                            newString = newString.replace(_searchStrings[filterNo],makeCorrectCaseSensitive(_searchStrings[filterNo], _cbCharacterType))
        else:
            newString = makeCorrectCaseSensitive(_inputString, _cbCharacterType)
        return newString
        
    def correctCaseSensitiveTable(_SpecialTools):
        from MyObjects import trForUI
        import Tables
        Tables.isChangeHiddenColumn,Tables.isAskShowHiddenColumn=True,True
        searchStrings = str(_SpecialTools.characterState.leSearch.text()).split(";")
        if _SpecialTools.characterState.columns.currentIndex()==0:
            columns = list(range(0,Universals.MainWindow.Table.columnCount()))
        else:
            columns = [_SpecialTools.characterState.columns.currentIndex()-1]
        for columnNo in columns:
            if Tables.checkHiddenColumn(columnNo,False)==False:
                continue
            for rowNo in range(Universals.MainWindow.Table.rowCount()):
                if Universals.MainWindow.Table.isChangableItem(rowNo, columnNo):
                    newString = str(Universals.MainWindow.Table.item(rowNo,columnNo).text())
                    myString = ""
                    informationSectionX = _SpecialTools.cbInformationSectionX.value()
                    informationSectionY = _SpecialTools.cbInformationSectionY.value()
                    cbCharacterType = Variables.validSentenceStructureKeys[_SpecialTools.characterState.cbCharacterType.currentIndex()]
                    isCaseSensitive = _SpecialTools.characterState.cckbCaseSensitive.isChecked()
                    isRegExp = _SpecialTools.characterState.cckbRegExp.isChecked()
                    isCorrectText = _SpecialTools.characterState.cckbCorrectText.isChecked()
                    if _SpecialTools.cbInformationSection.currentIndex()==0:
                        myString = correctCaseSensitive(newString, cbCharacterType, isCorrectText, searchStrings, isCaseSensitive, isRegExp)
                    elif _SpecialTools.cbInformationSection.currentIndex()==1:
                        myString = correctCaseSensitive(newString[:informationSectionX], cbCharacterType, isCorrectText, searchStrings, isCaseSensitive, isRegExp)
                        myString += newString[informationSectionX:]
                    elif _SpecialTools.cbInformationSection.currentIndex()==2:
                        myString = newString[:informationSectionX]
                        myString += correctCaseSensitive(newString[informationSectionX:], cbCharacterType, isCorrectText, searchStrings, isCaseSensitive, isRegExp)
                    elif _SpecialTools.cbInformationSection.currentIndex()==3:
                        myString = correctCaseSensitive(newString[:-informationSectionX], cbCharacterType, isCorrectText, searchStrings, isCaseSensitive, isRegExp)
                        myString += newString[-informationSectionX:]
                    elif _SpecialTools.cbInformationSection.currentIndex()==4:
                        myString = newString[:-informationSectionX]
                        myString += correctCaseSensitive(newString[-informationSectionX:], cbCharacterType, isCorrectText, searchStrings, isCaseSensitive, isRegExp)
                    elif _SpecialTools.cbInformationSection.currentIndex()==5:
                        myString = newString[:informationSectionX]
                        myString += correctCaseSensitive(newString[informationSectionX:informationSectionY], cbCharacterType, isCorrectText, searchStrings, isCaseSensitive, isRegExp)
                        myString += newString[informationSectionY:]
                    elif _SpecialTools.cbInformationSection.currentIndex()==6:
                        myString = correctCaseSensitive(newString[:informationSectionX], cbCharacterType, isCorrectText, searchStrings, isCaseSensitive, isRegExp)
                        myString += newString[informationSectionX:informationSectionY]
                        myString += correctCaseSensitive(newString[informationSectionY:], cbCharacterType, isCorrectText, searchStrings, isCaseSensitive, isRegExp)
                    Universals.MainWindow.Table.item(rowNo,columnNo).setText(trForUI(myString))
            
            
            



## This file is part of HamsiManager.
## 
## Copyright (c) 2010 Murat Demir <mopened@gmail.com>      
##
## Hamsi Manager is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## Hamsi Manager is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with HamsiManager; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import sys
if float(sys.version[:3])>=2.6:
    import sqlite3 as sqlite
else:
    from pysqlite2 import dbapi2 as sqlite
import Universals
from Databases import *
    
class Databases:
    global defaultConnection, getDefaultConnection, getAllDatabases, getDBPropertiesCreateQuery, reFillDatabases, correctForSql, getAmendedSQLInsertOrUpdateQueries, checkDatabases, getAmendedSQLSelectOrInsertAndSelectQueries
    defaultConnection = None
        
    def getDefaultConnection():
        global defaultConnection
        if defaultConnection==None:
            defaultConnection = sqlite.connect(Universals.pathOfSettingsDirectory + "/database.sqlite")
        return defaultConnection
    
    def getAllDatabases():
        return [BookmarksOfDirectories, BookmarksOfSpecialTools, SearchAndReplaceTable, CompleterTable]
    
    def getDBPropertiesCreateQuery():
        return "CREATE TABLE IF NOT EXISTS dbProperties ('keyName' TEXT NOT NULL,'value' TEXT)"
        
    def reFillDatabases(_table="All", _actionType="dropAndInsert", _makeBackUp=False):
        if _makeBackUp==True:
            makeBackUp(_table)
        tableCreateQueries, sqlCommands, tableInsertImportantQueries = [], [], []
        tableCreateQueries.append(getDBPropertiesCreateQuery())
        for database in getAllDatabases():
            if _table==database.tableName or _table=="All":
                tableCreateQueries.append(database.getTableCreateQuery())
                if _actionType=="dropAndInsert":
                    sqlCommands.append(database.getDeleteTableQuery())
                sqlCommands += database.getDefaultsQueries()
                tableInsertImportantQueries += getAmendedSQLInsertOrUpdateQueries("dbProperties", {"keyName" : "'" + database.tableName + "_Version'", "value" : "'" + str(database.tableVersion) + "'"}, ["keyName"])
        con = getDefaultConnection()
        for sqlCommand in tableCreateQueries:
            cur = con.cursor()
            cur.execute(str(sqlCommand))
            con.commit()
        for sqlCommand in tableInsertImportantQueries:
            cur = con.cursor()
            cur.execute(str(sqlCommand))
            con.commit()
        for sqlCommand in sqlCommands:
            cur = con.cursor()
            cur.execute(str(sqlCommand))
            con.commit()
    
    def correctForSql(_string):
        return str(_string).replace("'", "''")
    
    def getAmendedSQLInsertOrUpdateQueries(_table, _columnNamesAndValues, _primaryColumns):
        sqlString0 = "INSERT INTO " + _table + "("
        sqlString1 = ") SELECT "
        sqlString2 = " WHERE (SELECT COUNT(*) FROM " + _table + " WHERE "
        sqlString3 = "UPDATE " + _table + " SET "
        sqlString4 = " WHERE "
        i, j = 0, 0
        for key, value in _columnNamesAndValues.items():
            if i>0:
                sqlString0 += ","
                sqlString1 += ","
                sqlString3 += ","
            sqlString0 += key
            sqlString1 += str(value)
            sqlString3 += key + "=" + str(value)
            if _primaryColumns.count(key)>0:
                if j>0:
                    sqlString2 += " AND "
                    sqlString4 += " AND "
                sqlString2 += key + "=" + str(value)
                sqlString4 += key + "=" + str(value)
                j +=1
            i +=1
        return [sqlString0 + sqlString1 + sqlString2 + ")=0;" , sqlString3 + sqlString4 + ";"]
    
    def getAmendedSQLSelectOrInsertAndSelectQueries(_table, _selectedColumn, _columnNamesAndValues):
        sqlString0 = "SELECT " + _selectedColumn + " FROM " + _table + " WHERE " 
        sqlString1 = "INSERT INTO " + _table + "("
        sqlString2 = ") VALUES ("
        sqlString3 = ") ON DUPLICATE KEY UPDATE " 
        j = 0
        for key, value in _columnNamesAndValues.items():
            if j>0:
                sqlString1 += ","
                sqlString2 += ","
            sqlString1 += key
            sqlString2 += str(value)
            if j>0:
                sqlString0 += " AND "
                sqlString3 += " , "
            sqlString0 += key + "=" + str(value)
            sqlString3 += key + "=" + key
            j +=1
        return sqlString1 + sqlString2 + sqlString3 + ";" + sqlString0
        
    def checkDatabases():
        try:
            con = getDefaultConnection()
            cur = con.cursor()
            cur.execute("SELECT * FROM dbProperties")
            tableCreateQueries, sqlCommands, tableInsertImportantQueries = [], [], []
            for database in getAllDatabases():
                try:
                    cur = con.cursor()
                    cur.execute("SELECT * FROM dbProperties where keyName='" + database.tableName + "_Version'")
                    tableVersion = int(cur.fetchall()[0][1])
                except:
                    tableVersion = 0
                if tableVersion<database.tableVersion:
                    tableCreateQueries.append(database.getTableCreateQuery())
                    sqlCommands += database.getDefaultsQueries()
                    tableInsertImportantQueries += getAmendedSQLInsertOrUpdateQueries("dbProperties", {"keyName" : "'" + database.tableName + "_Version'", "value" : "'" + str(database.tableVersion) + "'"}, ["keyName"])
                    
            for sqlCommand in tableCreateQueries:
                cur = con.cursor()
                cur.execute(str(sqlCommand))
                con.commit()
            for sqlCommand in tableInsertImportantQueries:
                cur = con.cursor()
                cur.execute(str(sqlCommand))
                con.commit()
            for sqlCommand in sqlCommands:
                cur = con.cursor()
                cur.execute(str(sqlCommand))
                con.commit()
                
#            try:
#                cur.execute("SELECT * FROM dbProperties where keyName='bookmarksOfDirectories_Version'")
#                bookmarksOfDirectoriesVersion = int(cur.fetchall()[0][1])
#            except:
#                bookmarksOfDirectoriesVersion = 0
#            try:
#                cur.execute("SELECT * FROM dbProperties where keyName='bookmarksOfSpecialTools_Version'")
#                bookmarksOfSpecialToolsVersion = int(cur.fetchall()[0][1])
#            except:
#                bookmarksOfSpecialToolsVersion = 0
#            try:
#                cur.execute("SELECT * FROM dbProperties where keyName='searchAndReplaceTable_Version'")
#                searchAndReplaceTableVersion = int(cur.fetchall()[0][1])
#            except:
#                searchAndReplaceTableVersion = 0
#            if bookmarksOfDirectoriesVersion<x:
#                pass
#            if bookmarksOfSpecialToolsVersion<x:
#                pass
#            if searchAndReplaceTableVersion<x:
#                pass
        except:
            reFillDatabases()
    
        


